/*-------------------------------------------------------------------------
**
**                    Copyright (c) 1994 James R. Dyvig
** 
** Permission to use, copy, modify, and distribute this software and its
** documentation for educational, research, and non-profit purposes
** without fee is hereby granted, provided that the above copyright notice
** appear in all copies and that both that copyright notice and this
** permission notice appear in supporting documentation.
** 
** Permission to incorporate this software into commercial products can be
** obtained with the written consent of the author.
** 
** This software is provided AS IS with no warranties of any kind.  The
** author shall have no liability with respect to the infringement of
** copyrights, trade secrets or any patents by this file or any part
** thereof.  In no event will the author be liable for any lost revenue or
** profits or other special, indirect and consequential damages.
**
**-------------------------------------------------------------------------
*/
#include <Xm/Form.h>
#include <Xm/RowColumn.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/List.h>
#include <Xm/PushB.h>
#include <Xm/Separator.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include "regform.h"

extern RegForm	regform;
extern Boolean	use_registration;

/*-----------------------------------------------------------------
** Function: 	create_input_form
**
** Purpose:	Creates the basic input registration form
*/

void create_input_form(Widget workArea)
{
	int		ac = 0;
	Arg		al[30];
	char		buffer[80];
	Widget		button;
	Widget		form;
	Widget		frame;
	Widget		frame_box;
	Widget		label;
	XmString	label_string;
	Widget		rowcol;
	Widget		separator;
	Widget		target_list;
	Widget		text;
	Widget		title_label;
	XmString	title_string;
	Widget		workarea;
	extern void	make_frame(char *name, Widget form, Widget *frame_box,
		Widget *frame);
	extern void	textCB(Widget, char **, XmAnyCallbackStruct *cbs);
	extern void	sendCB(Widget, XtPointer, XmAnyCallbackStruct *cbs);
	extern Widget	make_labeled_text(Widget, char *, char *, int,
		char *, XtCallbackProc, XtPointer);
/*
** Create the background frame
*/
	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNmarginWidth,	5); ac++;
	XtSetArg(al[ac], XmNmarginHeight,	5); ac++;

	frame = XmCreateFrame(workArea, "frame", al, ac);
	XtManageChild(frame);
/*
** Create the background form
*/
	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_FORM); ac++;

	form = XmCreateForm(frame, "form", al, ac);
	XtManageChild(form);
/*
** Create title label
*/
	sprintf(buffer, "%s %s", regform.codename, regform.version);
	title_string = XmStringCreateSimple(buffer);

	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrecomputeSize,	False); ac++;
	XtSetArg(al[ac], XmNlabelString,	title_string); ac++;

	title_label = XmCreateLabel(form, "titlelabel", al, ac);
	XtManageChild(title_label);

	XmStringFree(title_string);
/*
** Add separator
*/
	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNtopWidget,		title_label); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_FORM); ac++;

	separator = XmCreateSeparator(form, "separator", al, ac);
	XtManageChild(separator);
/*
** Add Registration information box
*/
	if (use_registration) {
		make_frame("Registration Information", form, &frame_box,
			&workarea);
	} else {
		make_frame("User Information", form, &frame_box,
		&workarea);
	}

	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNtopWidget,		separator); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_FORM); ac++;

	XtSetValues(frame_box, al, ac);
/*
** Place all input information into a row column container
*/
	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNadjustLast,		True); ac++;

	rowcol = XmCreateRowColumn(workarea, "textfield", al, ac);
	XtManageChild(rowcol);

	make_labeled_text(rowcol, "input", "Name", 40, NULL,
		(XtCallbackProc) textCB, (XtPointer) &regform.name);
	
	if (use_registration) {
		make_labeled_text(rowcol, "input", "Company", 40, NULL,
			(XtCallbackProc) textCB, (XtPointer) &regform.company);
		make_labeled_text(rowcol, "input", "Address", 40, NULL,
			(XtCallbackProc) textCB, (XtPointer) &regform.address);
		make_labeled_text(rowcol, "input", "City, State, Zip", 40, NULL,
			(XtCallbackProc) textCB, (XtPointer) &regform.city);
		make_labeled_text(rowcol, "input", "Country", 40, NULL,
			(XtCallbackProc) textCB, (XtPointer) &regform.country);
	}
	make_labeled_text(rowcol, "input", "E-Mail", 40, NULL,
		(XtCallbackProc) textCB, (XtPointer) &regform.email);
/*
** Add separator
*/
	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNtopWidget,		frame_box); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_FORM); ac++;

	separator = XmCreateSeparator(form, "separator", al, ac);
	XtManageChild(separator);
/*
** Create the Survey Box
*/
	if (use_registration) {
		make_frame("Survey", form, &frame_box, &workarea);

		ac = 0;
		XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_WIDGET); ac++;
		XtSetArg(al[ac], XmNtopWidget,		separator); ac++;
		XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
		XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_FORM); ac++;

		XtSetValues(frame_box, al, ac);
/*
** Place all input information into a row column container
*/
		ac = 0;
		XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_FORM); ac++;
		XtSetArg(al[ac], XmNbottomAttachment,	XmATTACH_FORM); ac++;
		XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
		XtSetArg(al[ac], XmNadjustLast,		True); ac++;

		rowcol = XmCreateRowColumn(workarea, "textfield", al, ac);
		XtManageChild(rowcol);

		make_labeled_text(rowcol, "input", "Machine Types", 40, NULL,
			(XtCallbackProc) textCB, (XtPointer) &regform.machines);
		make_labeled_text(rowcol, "input", "Application Types", 40,
			NULL, (XtCallbackProc) textCB,
			(XtPointer) &regform.applications);
		make_labeled_text(rowcol, "input", "GUI Builder", 40, NULL,
			(XtCallbackProc) textCB,
			(XtPointer) &regform.gui_builders);
	}
/*
** Add separator
*/
	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNtopWidget,		frame_box); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_FORM); ac++;

	separator = XmCreateSeparator(form, "separator", al, ac);
	XtManageChild(separator);
/*
** Create the Remarks Box
*/
	make_frame("Comments", form, &frame_box, &workarea);

	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNtopWidget,		separator); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_FORM); ac++;

	XtSetValues(frame_box, al, ac);
/*
** Comment text field
*/
	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNcolumns,		60); ac++;
	XtSetArg(al[ac], XmNrows,		8); ac++;
	XtSetArg(al[ac], XmNeditMode,		XmMULTI_LINE_EDIT); ac++;

	text = XmCreateText(workarea, "textfield", al, ac);
	XtManageChild(text);

	XtAddCallback(text, XmNlosingFocusCallback,
		(XtCallbackProc) textCB, (XtPointer) &regform.comments);
/*
** Add separator
*/
	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNtopWidget,		frame_box); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_FORM); ac++;

	separator = XmCreateSeparator(form, "separator", al, ac);
	XtManageChild(separator);
/*
** Create the Destination box
*/
	make_frame("Destination", form, &frame_box, &workarea);

	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNtopWidget,		separator); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_FORM); ac++;

	XtSetValues(frame_box, al, ac);

	make_labeled_text(workarea, "input", "Send To", 40,
		regform.sendto,
		(XtCallbackProc) textCB, (XtPointer) &regform.sendto);
/*
** Add separator
*/
	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNtopWidget,		frame_box); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_FORM); ac++;

	separator = XmCreateSeparator(form, "separator", al, ac);
	XtManageChild(separator);
/*
** Create Send button
*/
	label_string = XmStringCreateSimple("Send");

	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNtopWidget,		separator); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNlabelString,	label_string); ac++;

	button = XmCreatePushButton(form, "button", al, ac);
	XtManageChild(button);

	XmStringFree(label_string);

	XtAddCallback(button, XmNactivateCallback, (XtCallbackProc) sendCB,
		NULL);
}
