/*-------------------------------------------------------------------------
**
**                    Copyright (c) 1994 James R. Dyvig
** 
** Permission to use, copy, modify, and distribute this software and its
** documentation for educational, research, and non-profit purposes
** without fee is hereby granted, provided that the above copyright notice
** appear in all copies and that both that copyright notice and this
** permission notice appear in supporting documentation.
** 
** Permission to incorporate this software into commercial products can be
** obtained with the written consent of the author.
** 
** This software is provided AS IS with no warranties of any kind.  The
** author shall have no liability with respect to the infringement of
** copyrights, trade secrets or any patents by this file or any part
** thereof.  In no event will the author be liable for any lost revenue or
** profits or other special, indirect and consequential damages.
**
**-------------------------------------------------------------------------
*/
#include <stdio.h>
#include <stdlib.h>
#include <Xm/MessageB.h>

#include "utils.h"
#include "regform.h"

extern RegForm	regform;
extern Boolean	use_registration;

/*-----------------------------------------------------------------
** Function:
**	really_quitCB
**
*/

void really_quitCB(Widget w, XtPointer value, XmAnyCallbackStruct *cbs)
{
	Widget			topLevel;
/*
** Locate the top level shell and destroy it.  This is not necessary,
** but it does keep our foot in the door for better cleanup policies
** in the future.
*/
	topLevel = w;
	while (topLevel && !XtIsTopLevelShell(topLevel))
		topLevel = XtParent(topLevel);
	
	XtDestroyWidget(topLevel);

	exit(0);
}

/*-----------------------------------------------------------------
** Function:
**	quitCB
**
*/

void quitCB(Widget w, XtPointer value, XmAnyCallbackStruct *cbs)
{
	extern void		really_quitCB(Widget w, XtPointer value,
					XmAnyCallbackStruct *cbs);

	really_quitCB(w, NULL, NULL);
}

/*-----------------------------------------------------------------
** Function:
**	version_helpCB
**
*/

#include "version_help.h"

void version_helpCB(Widget w, XtPointer value, XmAnyCallbackStruct *cbs)
{
	extern char	version_help_text[];

	show_text_block(w, REGFORM_NAME, "HELP:  Version",
		version_help_text);
}

/*-----------------------------------------------------------------
** Function:
**	overview_helpCB
**
*/

#include "overview_help.h"

void overview_helpCB(Widget w, XtPointer value, XmAnyCallbackStruct *cbs)
{
	extern char	overview_help_text[];

	show_text_block(w, REGFORM_NAME, "HELP:  Overview",
		overview_help_text);
}

/*-----------------------------------------------------------------
** Function:
**	textCB
**
*/

void textCB(Widget w, char **char_string, XmAnyCallbackStruct *cbs)
{
	char	buffer[200];
	int	count;

	XtVaGetValues(w,
		XmNvalue,	char_string,
		NULL);
	
	count = sscanf(*char_string, "%s", buffer);

	if (count < 1) *char_string = NULL;
}

/*-----------------------------------------------------------------
** Function:
**	sendCB
**
*/

void sendCB(Widget w, XtPointer value, XmAnyCallbackStruct *cbs)
{
	char	buffer[200];
	int	count;
	char	descriptor[80];
	char	filename[120];
	char	tmp_filename[80];
	FILE	*fp;
	char	*home;
	char	namecheck[80];
/*
** Open registration file
*/
	if (use_registration) {
		strcpy(tmp_filename, "Registration");
	} else {
		strcpy(tmp_filename, "Comment");
	}
	fp = fopen(tmp_filename, "w");
	if (!fp) {
		sprintf(buffer, "Could not open file: %s", tmp_filename);
		warning(w, buffer);
		return;
	}
/*
** Write header
*/
	fprintf(fp, "----------------------------------------------\n");
	fprintf(fp, "               %s %s\n", REGFORM_NAME, REGFORM_VERSION);
	fprintf(fp, "----------------------------------------------\n");
	fprintf(fp, "Purpose:		%s\n", tmp_filename);
/*
** Write the registration information
*/
	fprintf(fp, "Codename:		%s\n", regform.codename);
	fprintf(fp, "Version:		%s\n", regform.version);

	if (regform.name != NULL) {
		fprintf(fp, "Name:			%s\n", regform.name);
	} else {
		warning(w, "Please enter your name");
		return;
	}

	if (use_registration) {
		if (regform.company != NULL)
			fprintf(fp, "Company:		%s\n", regform.company);

		if (regform.address != NULL) {
			fprintf(fp, "Address:		%s\n", regform.address);
		} else {
			warning(w, "Please enter your address");
			return;
		}

		if (regform.city != NULL) {
			fprintf(fp, "City/State/Zip:		%s\n",
				regform.city);
		} else {
			warning(w, "Please enter your city, state and zip");
			return;
		}

		if (regform.country != NULL)
			fprintf(fp, "Country:		%s\n", regform.country);
	}

	if (regform.email != NULL) {
		fprintf(fp, "E-Mail:			%s\n", regform.email);
	} else {
		warning(w, "Please enter your e-mail address");
		return;
	}

	fprintf(fp, "\n");

	if (use_registration) {
		if (regform.machines != NULL)
			fprintf(fp, "Machines:		%s\n",
				regform.machines);

		if (regform.applications != NULL)
			fprintf(fp, "Applications:		%s\n",
				regform.applications);

		if (regform.gui_builders != NULL)
			fprintf(fp, "GUI_Builders:		%s\n",
				regform.gui_builders);

		fprintf(fp, "\n");
	}

	if (regform.comments != NULL)
		fprintf(fp, "Comments:\n%s\n", regform.comments);
/*
** Clean up and and send off
*/
	fclose(fp);

	printf("mail %s < %s\n", regform.sendto, tmp_filename);
	sprintf(buffer, "mail %s < %s", regform.sendto, tmp_filename);
	system(buffer);
/*
** Now that the important stuff is done, lets add this application
** to the user's regform history, BUT only if we are registering
** an application
*/
	if (! use_registration) return;
/*
** Create the name of the history file.  This assumes the user has
** a home variable $HOME
*/
	home = getenv("HOME");
	if (!home) {
		warning(w,"No home directory found\n");
		return;
	}

	sprintf(filename, "%s/.regform", home);
/*
** Check for duplicates
*/
	sprintf(namecheck, "Application: %s %s\n",
		regform.codename, regform.version);

	fp = fopen(filename, "r");
	if (fp) {
		while (fgets(buffer, 80, fp)) {
			if (strcmp(buffer, namecheck) == 0) {
				fclose(fp);
				return;
			}
		}

		fclose(fp);
	}
/*
** Now that we've checked for dups, add the application name to the
** history file
*/
	fp = fopen(filename, "a");
	if (!fp) {
		sprintf(buffer, "Could not open %s", filename);
		warning(w,buffer);

		sprintf(buffer, "Please e-mail %s", regform.sendto);
		warning(w,buffer);

		return;
	}

	fprintf(fp, "Application: %s %s\n", regform.codename, regform.version);

	fclose(fp);
}
