/*-------------------------------------------------------------------------
**
**                    Copyright (c) 1994 James R. Dyvig
** 
** Permission to use, copy, modify, and distribute this software and its
** documentation for educational, research, and non-profit purposes
** without fee is hereby granted, provided that the above copyright notice
** appear in all copies and that both that copyright notice and this
** permission notice appear in supporting documentation.
** 
** Permission to incorporate this software into commercial products can be
** obtained with the written consent of the author.
** 
** This software is provided AS IS with no warranties of any kind.  The
** author shall have no liability with respect to the infringement of
** copyrights, trade secrets or any patents by this file or any part
** thereof.  In no event will the author be liable for any lost revenue or
** profits or other special, indirect and consequential damages.
**
**-------------------------------------------------------------------------
*/
#include <stdio.h>
#include <Xm/ToggleB.h>
#include <Xm/Text.h>

#include "ib_prefs.h"

/*-----------------------------------------------------------------
** Function:
**	new_ib_prefs
**
*/

ib_prefs *new_ib_prefs(Widget workarea)
{
	ib_prefs	*p;

	p = (ib_prefs *) malloc(sizeof(ib_prefs));

	p->use_motif_1_2 = True;

	strcpy(p->cflags, "");
	strcpy(p->fflags, "");
	strcpy(p->customflags, "");

	strcpy(p->sys_includes, "");

	p->compiler = IB_CC;
	strcpy(p->compiler_text, "");

	strcpy(p->config_location, "/usr/lib/X11/config");

	p->bootstrap = IB_IMAKE;
	strcpy(p->bootstrap_imake_text,
		"-DUseInstalled -DBuildMotif -DUseInstalledMotif");
	strcpy(p->bootstrap_xmkmf_text, "");
	strcpy(p->bootstrap_custom_text, "");

	strcpy(p->file_match, "*.[cfC]*");

	strcpy(p->build_string, "");
	strcpy(p->make_exe, "");

	load_prefs_file(p);

	p->gui = new_ib_prefs_gui(workarea);

	set_ib_prefs_gui(p);

	return(p);
}

/*-----------------------------------------------------------------
** Function:
**	zap_ib_prefs
**
*/

void zap_ib_prefs(ib_prefs *p)
{
	zap_ib_prefs_gui(p->gui);
	free(p);
}

/*-----------------------------------------------------------------
** Function:
**	set_ib_prefs_gui
**
*/

void set_ib_prefs_gui(ib_prefs *p)
{
	ib_prefs_gui	*gui;

	if (p->gui->top_level == NULL) return;

	gui = p->gui;
/*
** Set Motif version
*/
	if (p->use_motif_1_2) {
		XmToggleButtonSetState(gui->motif_1_1, False, False);
		XmToggleButtonSetState(gui->motif_1_2, True, False);
	} else {
		XmToggleButtonSetState(gui->motif_1_1, True, False);
		XmToggleButtonSetState(gui->motif_1_2, False, False);
	}
/*
** Set Compiler
*/
	switch (p->compiler) {
	case IB_CC:
		XmToggleButtonSetState(gui->compiler_cc, True, False);
		XmToggleButtonSetState(gui->compiler_gcc, False, False);
		XmToggleButtonSetState(gui->compiler_gcpp, False, False);
		XmToggleButtonSetState(gui->compiler_custom, False, False);
		break;
	case IB_GCC:
		XmToggleButtonSetState(gui->compiler_cc, False, False);
		XmToggleButtonSetState(gui->compiler_gcc, True, False);
		XmToggleButtonSetState(gui->compiler_gcpp, False, False);
		XmToggleButtonSetState(gui->compiler_custom, False, False);
		break;
	case IB_GCPP:
		XmToggleButtonSetState(gui->compiler_cc, False, False);
		XmToggleButtonSetState(gui->compiler_gcc, False, False);
		XmToggleButtonSetState(gui->compiler_gcpp, True, False);
		XmToggleButtonSetState(gui->compiler_custom, False, False);
		break;
	case IB_CUSTOM_CC:
		XmToggleButtonSetState(gui->compiler_cc, False, False);
		XmToggleButtonSetState(gui->compiler_gcc, False, False);
		XmToggleButtonSetState(gui->compiler_gcpp, False, False);
		XmToggleButtonSetState(gui->compiler_custom, True, False);
		XmTextSetString(gui->compiler_custom_text, p->compiler_text);
		break;
	default:
		XmToggleButtonSetState(gui->compiler_cc, True, False);
		XmToggleButtonSetState(gui->compiler_gcc, False, False);
		XmToggleButtonSetState(gui->compiler_gcpp, False, False);
		XmToggleButtonSetState(gui->compiler_custom, False, False);
		break;
	}
/*
** Set Compiler flags
*/
	XmTextSetString(gui->cflags, p->cflags);
	XmTextSetString(gui->fflags, p->fflags);
	XmTextSetString(gui->customflags, p->customflags);
/*
** Set System Includes
*/
	XmTextSetString(gui->sys_includes, p->sys_includes);
/*
** Set Bootstrap
*/
	switch(p->bootstrap) {
	case IB_IMAKE:
		XmToggleButtonSetState(gui->bootstrap_imake, True, False);
		XmToggleButtonSetState(gui->bootstrap_xmkmf, False, False);
		XmToggleButtonSetState(gui->bootstrap_custom, False, False);
		XmTextSetString(gui->bootstrap_imake_text,
			p->bootstrap_imake_text);
		break;
	case IB_XMKMF:
		XmToggleButtonSetState(gui->bootstrap_imake, False, False);
		XmToggleButtonSetState(gui->bootstrap_xmkmf, True, False);
		XmToggleButtonSetState(gui->bootstrap_custom, False, False);
		XmTextSetString(gui->bootstrap_xmkmf_text,
			p->bootstrap_xmkmf_text);
		break;
	case IB_CUSTOM:
		XmToggleButtonSetState(gui->bootstrap_imake, False, False);
		XmToggleButtonSetState(gui->bootstrap_xmkmf, False, False);
		XmToggleButtonSetState(gui->bootstrap_custom, True, False);
		XmTextSetString(gui->bootstrap_custom_text,
			p->bootstrap_custom_text);
		break;
	default:
		XmToggleButtonSetState(gui->bootstrap_imake, True, False);
		XmToggleButtonSetState(gui->bootstrap_xmkmf, False, False);
		XmToggleButtonSetState(gui->bootstrap_custom, False, False);
		XmTextSetString(gui->bootstrap_imake_text,
			p->bootstrap_imake_text);
		break;
	}
/*
** Set config directory
*/
	XmTextSetString(gui->config_location, p->config_location);
}

/*-----------------------------------------------------------------
** Function:
**	get_ib_prefs_gui
**
*/

void get_ib_prefs_gui(ib_prefs *p)
{
	ib_prefs_gui	*gui;
	char		*text;

	if (p->gui->top_level == NULL) return;

	gui = p->gui;
/*
** Check Motif version
*/
	p->use_motif_1_2 = XmToggleButtonGetState(gui->motif_1_2);
/*
** Get Compiler
*/
	if (XmToggleButtonGetState(gui->compiler_gcc)) {
		p->compiler = IB_GCC;
	} else if (XmToggleButtonGetState(gui->compiler_gcpp)) {
		p->compiler = IB_GCPP;
	} else if (XmToggleButtonGetState(gui->compiler_custom)) {
		p->compiler = IB_CUSTOM_CC;
		text = XmTextGetString(gui->compiler_custom_text);
		strcpy(p->compiler_text, text);
		XtFree(text);
	} else {
		p->compiler = IB_CC;
	}
/*
** Get Compiler flags
*/
	text = XmTextGetString(gui->cflags);
	strcpy(p->cflags, text);
	XtFree(text);

	text = XmTextGetString(gui->fflags);
	strcpy(p->fflags, text);
	XtFree(text);

	text = XmTextGetString(gui->customflags);
	strcpy(p->customflags, text);
	XtFree(text);
/*
** Get System Includes
*/
	text = XmTextGetString(gui->sys_includes);
	strcpy(p->sys_includes, text);
	XtFree(text);
/*
** Get Bootstrap
*/
	if (XmToggleButtonGetState(gui->bootstrap_imake)) {
		p->bootstrap = IB_IMAKE;
		text = XmTextGetString(gui->bootstrap_imake_text);
		strcpy(p->bootstrap_imake_text, text);
		XtFree(text);
	} else if (XmToggleButtonGetState(gui->bootstrap_custom)) {
		p->bootstrap = IB_CUSTOM;
		text = XmTextGetString(gui->bootstrap_custom_text);
		strcpy(p->bootstrap_custom_text, text);
		XtFree(text);
	} else {
		p->bootstrap = IB_XMKMF;
		text = XmTextGetString(gui->bootstrap_xmkmf_text);
		strcpy(p->bootstrap_xmkmf_text, text);
		XtFree(text);
	}
/*
** Get config directory
*/
	text = XmTextGetString(gui->config_location);
	strcpy(p->config_location, text);
	XtFree(text);
}

/*-----------------------------------------------------------------
** Function:
**	load_prefs_file
**
*/

void load_prefs_file(ib_prefs *p)
{
	FILE		*fp;
	char		buffer[MAXCHAR];
	char		*filename;
	char		default_file[MAXCHAR];
	char		descript[MAXCHAR];
	int		num;
	char		value[MAXCHAR];
	char		morevalues[MAXCHAR];
	extern char	*get_file_path(char *env_var, char *filename);
/*
** Determine home directory
*/
	strcpy(default_file, ".ib_config");
	filename = get_file_path("HOME", default_file);

	if (filename == NULL) return;
/*
** Tack on config file name and open
*/
	fp = fopen(filename, "r");
	if (!fp) {
		return;
	}
/*
** Read header
*/
	fgets(buffer, MAXCHAR, fp);
	fgets(buffer, MAXCHAR, fp);
/*
** Motif version
*/
	fgets(buffer, MAXCHAR, fp);
	sscanf(buffer, "%s %s", descript, value);

	if (strcmp(value, "1.1") == 0) {
		p->use_motif_1_2 = False;
	} else {
		p->use_motif_1_2 = True;
	}
/*
** Compiler
*/
	fgets(buffer, MAXCHAR, fp);
	sscanf(buffer, "%s %s", descript, value);

	if (strcmp(value, "cc") == 0) {
		p->compiler = IB_CC;
	} else if (strcmp(value, "gcc") == 0) {
		p->compiler = IB_GCC;
	} else if (strcmp(value, "g++") == 0) {
		p->compiler = IB_GCPP;
	} else {
		p->compiler = IB_CUSTOM_CC;
		num = sscanf(buffer,"%s %s \"%[^\"]\"", descript, value,
			p->compiler_text);
		if (num != 3) strcpy(p->compiler_text, "");
	}
/*
** Compiler Flags
*/
	fgets(buffer, MAXCHAR, fp);
	num = sscanf(buffer,"%s \"%[^\"]\"", descript, p->cflags);
	if (num != 2) strcpy(p->cflags, "");

	fgets(buffer, MAXCHAR, fp);
	num = sscanf(buffer,"%s \"%[^\"]\"", descript, p->fflags);
	if (num != 2) strcpy(p->fflags, "");

	fgets(buffer, MAXCHAR, fp);
	num = sscanf(buffer,"%s \"%[^\"]\"", descript, p->customflags);
	if (num != 2) strcpy(p->customflags, "");
/*
** Bootstrap
*/
	fgets(buffer, MAXCHAR, fp);
	num = sscanf(buffer,"%s %s \"%[^\"]\"", descript, value, morevalues);
	if (num != 3) strcpy(morevalues, "");

	if (strcmp(value, "imake") == 0) {
		p->bootstrap = IB_IMAKE;
		strcpy(p->bootstrap_imake_text, morevalues);
	} else if (strcmp(value, "xmkmf") == 0) {
		p->bootstrap = IB_XMKMF;
		strcpy(p->bootstrap_xmkmf_text, morevalues);
	} else {
		p->bootstrap = IB_CUSTOM;
		strcpy(p->bootstrap_custom_text, morevalues);
	}
/*
** Config Dir
*/
	fgets(buffer, MAXCHAR, fp);
	sscanf(buffer,"%s %s", descript, p->config_location);
/*
** System Includes
*/
	fgets(buffer, MAXCHAR, fp);
	num = sscanf(buffer,"%s \"%[^\"]\"", descript, p->sys_includes);
	if (num != 2) strcpy(p->sys_includes, "");
/*
** Close and return
*/
	fclose(fp);
}
