/*-------------------------------------------------------------------------
**
**                    Copyright (c) 1994 James R. Dyvig
** 
** Permission to use, copy, modify, and distribute this software and its
** documentation for educational, research, and non-profit purposes
** without fee is hereby granted, provided that the above copyright notice
** appear in all copies and that both that copyright notice and this
** permission notice appear in supporting documentation.
** 
** Permission to incorporate this software into commercial products can be
** obtained with the written consent of the author.
** 
** This software is provided AS IS with no warranties of any kind.  The
** author shall have no liability with respect to the infringement of
** copyrights, trade secrets or any patents by this file or any part
** thereof.  In no event will the author be liable for any lost revenue or
** profits or other special, indirect and consequential damages.
**
**-------------------------------------------------------------------------
*/
#include <stdio.h>
#include <Xm/Text.h>
#include <Xm/MessageB.h>

#include "ib_project.h"
#include "ib_prefs.h"

/*-----------------------------------------------------------------
** Function:
**	prefs_ok_cb
**
*/

void prefs_ok_cb(Widget w, ib_prefs_gui *gui, XtPointer cbs)
{
	extern ib_project	*project;

	dimiss_ib_prefs_gui(gui);

	get_ib_prefs_gui(project->prefs);
}

/*-----------------------------------------------------------------
** Function:
**	prefs_cancel_cb
**
*/

void prefs_cancel_cb(Widget w, ib_prefs_gui *gui, XtPointer cbs)
{
	dimiss_ib_prefs_gui(gui);
}

/*-----------------------------------------------------------------
** Function:
**	prefs_save_cb
**
*/

void prefs_save_cb(Widget w, ib_prefs_gui *gui, XtPointer cbs)
{
	FILE			*fp;
	char			buffer[MAXCHAR];
	char			*filename;
	char			default_file[MAXCHAR];
	ib_prefs		*p;
	extern ib_project	*project;
	extern char		*get_file_path(char *env_var, char *filename);
	extern void		warning(Widget w, char *s);

	p = project->prefs;
/*
** Make sure we grab the current copy
*/
	get_ib_prefs_gui(p);
/*
** Determine home directory
*/
	strcpy(default_file, ".ib_config");

	filename = get_file_path("HOME", default_file);

	if (filename == NULL) filename = default_file;
/*
** Tack on config file name and open
*/
	fp = fopen(filename, "w");
	if (!fp) {
		sprintf(buffer,"Could not write to %s\n",filename);
		warning(project->gui->top_level, buffer);
		return;
	}
/*
** Write config file
*/
	fprintf(fp, "                               Imake Builder %s\n",
		IB_VERSION);
	fprintf(fp, "----------------------------------------");
	fprintf(fp, "---------------------------------------\n");
/*
** Motif version
*/
	fprintf(fp, "MotifVersion	");
	if (p->use_motif_1_2) {
		fprintf(fp, "1.2\n");
	} else {
		fprintf(fp, "1.1\n");
	}
/*
** Compiler
*/
	fprintf(fp, "Compiler	");
	switch (p->compiler) {
	case IB_CC:
		fprintf(fp, "cc\n");
		break;
	case IB_GCC:
		fprintf(fp, "gcc\n");
		break;
	case IB_GCPP:
		fprintf(fp, "g++\n");
		break;
	case IB_CUSTOM_CC:
		fprintf(fp, "%s\n", p->compiler_text);
		break;
	default:
		fprintf(fp, "cc\n");
		break;
	}
/*
** Compiler Flags
*/
	fprintf(fp, "CFLAGS		\"%s\"\n", p->cflags);
	fprintf(fp, "FFLAGS		\"%s\"\n", p->fflags);
	fprintf(fp, "CUSTOMFLAGS	\"%s\"\n", p->customflags);
/*
** Bootstrap
*/
	fprintf(fp, "Bootstrap	");
	switch(p->bootstrap) {
	case IB_IMAKE:
		fprintf(fp, "imake	\"%s\"\n", p->bootstrap_imake_text);
		break;
	case IB_XMKMF:
		fprintf(fp, "xmkmf	\"%s\"\n", p->bootstrap_xmkmf_text);
		break;
	case IB_CUSTOM:
		fprintf(fp, "custom	\"%s\"\n", p->bootstrap_custom_text);
		break;
	default:
		fprintf(fp, "xmkmf	\"%s\"\n", p->bootstrap_xmkmf_text);
		break;
	}
/*
** Config Dir
*/
	fprintf(fp, "ConfigLocation	%s\n", p->config_location);
/*
** System Includes
*/
	fprintf(fp, "SystemIncludes	\"%s\"\n", p->sys_includes);
/*
** Close and return
*/
	fclose(fp);
}
