/*-------------------------------------------------------------------------
**
**                    Copyright (c) 1994 James R. Dyvig
** 
** Permission to use, copy, modify, and distribute this software and its
** documentation for educational, research, and non-profit purposes
** without fee is hereby granted, provided that the above copyright notice
** appear in all copies and that both that copyright notice and this
** permission notice appear in supporting documentation.
** 
** Permission to incorporate this software into commercial products can be
** obtained with the written consent of the author.
** 
** This software is provided AS IS with no warranties of any kind.  The
** author shall have no liability with respect to the infringement of
** copyrights, trade secrets or any patents by this file or any part
** thereof.  In no event will the author be liable for any lost revenue or
** profits or other special, indirect and consequential damages.
**
**-------------------------------------------------------------------------
*/
#include <Xm/Form.h>
#include <Xm/Label.h>
#include <Xm/PushBG.h>
#include <Xm/RowColumn.h>
#include <Xm/Separator.h>
#include <Xm/Text.h>
#include <Xm/ToggleB.h>

#include "ib_prefs.h"

/*-----------------------------------------------------------------
** Function:
**	new_ib_prefs_gui
**
*/

ib_prefs_gui *new_ib_prefs_gui(Widget workarea)
{
	ib_prefs_gui	*p;

	p = (ib_prefs_gui *) malloc(sizeof(ib_prefs_gui));

	p->build_string = NULL;

	p->cflags = NULL;
	p->fflags = NULL;
	p->customflags = NULL;

	p->bootstrap_imake = NULL;
	p->bootstrap_xmkmf = NULL;
	p->bootstrap_custom = NULL;
	p->bootstrap_imake_text = NULL;
	p->bootstrap_xmkmf_text = NULL;
	p->bootstrap_custom_text = NULL;

	p->compiler_cc = NULL;
	p->compiler_gcc = NULL;
	p->compiler_gcpp = NULL;
	p->compiler_custom = NULL;
	p->compiler_custom_text = NULL;

	p->config_location = NULL;

	p->file_match = NULL;

	p->motif_1_1 = NULL;
	p->motif_1_2 = NULL;

	p->makefile = NULL;
	p->top_level = NULL;

	create_ib_prefs_gui(p, workarea);

	return(p);
}

/*-----------------------------------------------------------------
** Function:
**	show_ib_prefs_gui
**
*/

void show_ib_prefs_gui(ib_prefs_gui *p)
{
	if (p->top_level == NULL) {
		return;
	} else {
		XtRealizeWidget(p->top_level);
	}
}

/*-----------------------------------------------------------------
** Function:
**	dismiss_ib_prefs_gui
**
*/

void dimiss_ib_prefs_gui(ib_prefs_gui *p)
{
	if (p->top_level == NULL) {
		return;
	} else {
		XtUnrealizeWidget(p->top_level);
	}
}

/*-----------------------------------------------------------------
** Function:
**	zap_ib_prefs_gui
**
*/

void zap_ib_prefs_gui(ib_prefs_gui *p)
{
	if (p->top_level == NULL) {
		return;
	} else {
		XtDestroyWidget(p->top_level);
		p->top_level = NULL;
	}
}

/*-----------------------------------------------------------------
** Function:
**	create_ib_prefs_gui
**
*/

void create_ib_prefs_gui(ib_prefs_gui *gui, Widget workArea)
{
	int		ac;
	Arg		al[30];
	Widget		bootstrap_box;
	Widget		button;
	Widget		compiler_box;
	Widget		config_box;
	Widget		flags_box;
	Widget		form;
	Widget		rowcol;
	Widget		separator;
	Widget		shell;
	Widget		sys_include_box;
	Widget		top_level;
	Widget		version_box;
	extern Widget	pref_bootstrap_box(ib_prefs_gui *gui, Widget form);
	extern Widget	pref_config_box(ib_prefs_gui *gui, Widget form);
	extern Widget	pref_compiler_box(ib_prefs_gui *gui, Widget form);
	extern Widget	pref_flags_box(ib_prefs_gui *gui, Widget form);
	extern Widget	pref_sys_inc_box(ib_prefs_gui *gui, Widget form);
	extern Widget	pref_version_box(ib_prefs_gui *gui, Widget form);
	extern void	prefs_ok_cb(Widget w, ib_prefs_gui *gui, XtPointer x);
	extern void	prefs_cancel_cb(Widget w,ib_prefs_gui *gui,XtPointer x);
	extern void	prefs_save_cb(Widget w, ib_prefs_gui *gui, XtPointer x);

	if (gui->top_level != NULL) return;
/*
** Find top level shell to create new shell from
*/
	top_level = workArea;
	while (top_level && ! XtIsTopLevelShell(top_level))
		top_level = XtParent(top_level);
/*
** Create the new shell
*/
	shell = XtVaAppCreateShell("Preferences", "IB",
		topLevelShellWidgetClass, XtDisplay(top_level),
		NULL, 0);
/*
** Create a form to work from
*/
	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,      XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,   XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,     XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,    XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNmarginWidth,	4); ac++;
	XtSetArg(al[ac], XmNmarginHeight,       4); ac++;
	XtSetArg(al[ac], XmNfractionBase,       2); ac++;

	form = XmCreateForm(shell, "form", al, ac);
	XtManageChild(form);
/*
** Compiler Box
*/
	compiler_box = pref_compiler_box(gui, form);

	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,	XmATTACH_NONE); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_NONE); ac++;
	XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_FORM); ac++;

	XtSetValues(compiler_box, al, ac);
/*
** Motif Version Box
*/
	version_box = pref_version_box(gui, form);

	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,	XmATTACH_OPPOSITE_WIDGET);ac++;
	XtSetArg(al[ac], XmNbottomWidget,	compiler_box);ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNrightWidget,	compiler_box); ac++;

	XtSetValues(version_box, al, ac);
/*
** System Include Flags Box
*/
	sys_include_box = pref_sys_inc_box(gui, form);

	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNtopWidget,		version_box); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,	XmATTACH_NONE); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_FORM); ac++;

	XtSetValues(sys_include_box, al, ac);
/*
** Compiler Flags Box
*/
	flags_box = pref_flags_box(gui, form);

	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNtopWidget,		sys_include_box); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,	XmATTACH_NONE); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_FORM); ac++;

	XtSetValues(flags_box, al, ac);
/*
** Bootstraps Box
*/
	bootstrap_box = pref_bootstrap_box(gui, form);

	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNtopWidget,		flags_box); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,	XmATTACH_NONE); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_FORM); ac++;

	XtSetValues(bootstrap_box, al, ac);
/*
** Config Directory Box
*/
	config_box = pref_config_box(gui, form);

	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNtopWidget,		bootstrap_box); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,	XmATTACH_NONE); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_FORM); ac++;

	XtSetValues(config_box, al, ac);
/*
** Separator
*/
	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNtopWidget,		config_box); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,	XmATTACH_NONE); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_FORM); ac++;

	separator = XmCreateSeparator(form, "separator", al, ac);
	XtManageChild(separator);
/*
** Rowcol for OK, Cancel, and Save buttons
*/
	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNtopWidget,		separator); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNorientation,	XmHORIZONTAL); ac++;
	XtSetArg(al[ac], XmNadjustLast,		False); ac++;

	rowcol = XmCreateRowColumn(form, "rowcol", al, ac);
	XtManageChild(rowcol);
/*
** OK Button, Cancel, and Save buttons
*/
	ac = 0;
	button = XmCreatePushButtonGadget(rowcol, "OK", al, ac);
	XtManageChild(button);

	XtAddCallback(button, XmNactivateCallback,
		(XtCallbackProc)prefs_ok_cb, (XtPointer) gui);

	button = XmCreatePushButtonGadget(rowcol, "Cancel", al, ac);
	XtManageChild(button);

	XtAddCallback(button, XmNactivateCallback,
		(XtCallbackProc)prefs_cancel_cb, (XtPointer) gui);

	button = XmCreatePushButtonGadget(rowcol, "Save", al, ac);
	XtManageChild(button);

	XtAddCallback(button, XmNactivateCallback,
		(XtCallbackProc)prefs_save_cb, (XtPointer) gui);
/*
** Assign scenario structs
*/
	gui->top_level = shell;
}

/*-----------------------------------------------------------------
** Function:
**	pref_compiler_box
**
*/

Widget pref_compiler_box(ib_prefs_gui *gui, Widget form)
{
	int		ac;
	Arg		al[30];
	Widget		frame;
	Widget		frame_box;
	Widget		label;
	XmString	label_string;
	Widget		radio_box;
	Widget		rowcol;
	Widget		text;
	Widget		toggle;
	Widget		workarea;
	extern void	make_frame(char *name, Widget form, Widget *frame_box,
				Widget *frame);
/*
** Type box
*/
	make_frame("Compiler", form, &frame_box, &frame);

	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,      XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,   XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,     XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,    XmATTACH_FORM); ac++;

	workarea = XmCreateForm(frame, "form", al, ac);
	XtManageChild(workarea);
/*
** Radio box
*/
	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,      XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,   XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,     XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,    XmATTACH_NONE); ac++;

	radio_box = XmCreateRadioBox(workarea, "radiobox", al, ac);
	XtManageChild(radio_box);
/*
** CC
*/
	label_string = XmStringCreateSimple("System C Compiler (cc)");

	ac = 0;
	XtSetArg(al[ac], XmNlabelString,	label_string); ac++;

	toggle = XmCreateToggleButton(radio_box, "toggle", al, ac);
	XtManageChild(toggle);

	XmStringFree(label_string);

	gui->compiler_cc = toggle;
/*
** GCC
*/
	label_string = XmStringCreateSimple("Gnu C Compiler (gcc)");

	ac = 0;
	XtSetArg(al[ac], XmNlabelString,	label_string); ac++;

	toggle = XmCreateToggleButton(radio_box, "toggle", al, ac);
	XtManageChild(toggle);

	XmStringFree(label_string);

	gui->compiler_gcc = toggle;
/*
** GCPP
*/
	label_string = XmStringCreateSimple("Gnu C++ Compiler (g++)");

	ac = 0;
	XtSetArg(al[ac], XmNlabelString,	label_string); ac++;

	toggle = XmCreateToggleButton(radio_box, "toggle", al, ac);
	XtManageChild(toggle);

	XmStringFree(label_string);

	gui->compiler_gcpp = toggle;
/*
** Custom compiler
*/
	label_string = XmStringCreateSimple("Custom Compiler");

	ac = 0;
	XtSetArg(al[ac], XmNlabelString,	label_string); ac++;

	toggle = XmCreateToggleButton(radio_box, "toggle", al, ac);
	XtManageChild(toggle);

	XmStringFree(label_string);

	gui->compiler_custom = toggle;
/*
** Add input text for some of the toggles
*/
	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,      XmATTACH_NONE); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,   XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,     XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNleftWidget,     	radio_box); ac++;
	XtSetArg(al[ac], XmNrightAttachment,    XmATTACH_FORM); ac++;

	text = XmCreateText(workarea, "textfield", al, ac);
	XtManageChild(text);

	gui->compiler_custom_text = text;
/*
** Return boxed widget
*/
	return(frame_box);
}


/*-----------------------------------------------------------------
** Function:
**	pref_version_box
**
*/

Widget pref_version_box(ib_prefs_gui *gui, Widget form)
{
	int		ac;
	Arg		al[30];
	Widget		frame;
	Widget		frame_box;
	Widget		label;
	XmString	label_string;
	Widget		radio_box;
	Widget		toggle;
	Widget		workarea;
	extern void	make_frame(char *name, Widget form, Widget *frame_box,
				Widget *frame);
/*
** Version box
*/
	make_frame("Versions", form, &frame_box, &frame);

	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,      XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,   XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,     XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,    XmATTACH_FORM); ac++;

	workarea = XmCreateForm(frame, "form", al, ac);
	XtManageChild(workarea);
/*
** Radio box
*/
	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,      XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,   XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,     XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,    XmATTACH_FORM); ac++;

	radio_box = XmCreateRadioBox(workarea, "radiobox", al, ac);
	XtManageChild(radio_box);
/*
** Motif 1.1
*/
	label_string = XmStringCreateSimple("SGI Motif 1.1");

	ac = 0;
	XtSetArg(al[ac], XmNlabelString,	label_string); ac++;

	toggle = XmCreateToggleButton(radio_box, "toggle", al, ac);
	XtManageChild(toggle);

	XmStringFree(label_string);

	gui->motif_1_1 = toggle;
/*
** Motif 1.2
*/
	label_string = XmStringCreateSimple("Motif 1.2");

	ac = 0;
	XtSetArg(al[ac], XmNlabelString,	label_string); ac++;

	toggle = XmCreateToggleButton(radio_box, "toggle", al, ac);
	XtManageChild(toggle);

	XmStringFree(label_string);

	gui->motif_1_2 = toggle;
/*
** Return boxed widget
*/
	return(frame_box);
}


/*-----------------------------------------------------------------
** Function:
**	pref_sys_inc_box
**
*/

Widget pref_sys_inc_box(ib_prefs_gui *gui, Widget form)
{
	int		ac;
	Arg		al[30];
	Widget		frame;
	Widget		frame_box;
	Widget		label;
	XmString	label_string;
	Widget		rowcol;
	Widget		text;
	Widget		toggle;
	Widget		workarea;
	extern void	make_frame(char *name, Widget form, Widget *frame_box,
				Widget *frame);
/*
** Type box
*/
	make_frame("System Includes", form, &frame_box, &frame);

	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,      XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,   XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,     XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,    XmATTACH_FORM); ac++;

	workarea = XmCreateForm(frame, "form", al, ac);
	XtManageChild(workarea);
/*
** Rowcol
*/
	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,      XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,   XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,     XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,    XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNpacking,    	XmPACK_COLUMN); ac++;
	XtSetArg(al[ac], XmNnumColumns,    	2); ac++;
	XtSetArg(al[ac], XmNadjustLast,    	True); ac++;

	rowcol = XmCreateRowColumn(workarea, "rowcol", al, ac);
	XtManageChild(rowcol);
/*
** Label SYSTEM INCLUDES
*/
	label_string = XmStringCreateSimple("Include Directories");

	ac = 0;
	XtSetArg(al[ac], XmNlabelString,	label_string); ac++;

	label = XmCreateLabel(rowcol, "label", al, ac);
	XtManageChild(label);

	XmStringFree(label_string);
/*
** SYS INCLUDES Text Field
*/
	ac = 0;

	text = XmCreateText(rowcol, "textfield", al, ac);
	XtManageChild(text);

	gui->sys_includes = text;
/*
** Return boxed widget
*/
	return(frame_box);
}



/*-----------------------------------------------------------------
** Function:
**	pref_flags_box
**
*/

Widget pref_flags_box(ib_prefs_gui *gui, Widget form)
{
	int		ac;
	Arg		al[30];
	Widget		frame;
	Widget		frame_box;
	Widget		label;
	XmString	label_string;
	Widget		rowcol;
	Widget		text;
	Widget		toggle;
	Widget		workarea;
	extern void	make_frame(char *name, Widget form, Widget *frame_box,
				Widget *frame);
/*
** Type box
*/
	make_frame("Compiler Flags", form, &frame_box, &frame);

	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,      XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,   XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,     XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,    XmATTACH_FORM); ac++;

	workarea = XmCreateForm(frame, "form", al, ac);
	XtManageChild(workarea);
/*
** Rowcol
*/
	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,      XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,   XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,     XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,    XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNpacking,    	XmPACK_COLUMN); ac++;
	XtSetArg(al[ac], XmNnumColumns,    	2); ac++;
	XtSetArg(al[ac], XmNadjustLast,    	True); ac++;

	rowcol = XmCreateRowColumn(workarea, "rowcol", al, ac);
	XtManageChild(rowcol);
/*
** Label CCOPTIONS
*/
	label_string = XmStringCreateSimple("CCOPTIONS");

	ac = 0;
	XtSetArg(al[ac], XmNlabelString,	label_string); ac++;

	label = XmCreateLabel(rowcol, "label", al, ac);
	XtManageChild(label);

	XmStringFree(label_string);
/*
** Label FFLAGS
*/
	label_string = XmStringCreateSimple("FFLAGS");

	ac = 0;
	XtSetArg(al[ac], XmNlabelString,	label_string); ac++;

	label = XmCreateLabel(rowcol, "label", al, ac);
	XtManageChild(label);

	XmStringFree(label_string);
/*
** Label CUSTOM
*/
	label_string = XmStringCreateSimple("CUSTOM");

	ac = 0;
	XtSetArg(al[ac], XmNlabelString,	label_string); ac++;

	label = XmCreateLabel(rowcol, "label", al, ac);
	XtManageChild(label);

	XmStringFree(label_string);
/*
** CCOPTIONS Text Field
*/
	ac = 0;

	text = XmCreateText(rowcol, "textfield", al, ac);
	XtManageChild(text);

	gui->cflags = text;
/*
** FFLAGS Text Field
*/
	ac = 0;

	text = XmCreateText(rowcol, "textfield", al, ac);
	XtManageChild(text);

	gui->fflags = text;
/*
** CUSTOM Text Field
*/
	ac = 0;

	text = XmCreateText(rowcol, "textfield", al, ac);
	XtManageChild(text);

	gui->customflags = text;
/*
** Return boxed widget
*/
	return(frame_box);
}


/*-----------------------------------------------------------------
** Function:
**	pref_bootstrap_box
**
*/

Widget pref_bootstrap_box(ib_prefs_gui *gui, Widget form)
{
	int		ac;
	Arg		al[30];
	Widget		frame;
	Widget		frame_box;
	Widget		label;
	XmString	label_string;
	Widget		radio_box;
	Widget		rowcol;
	Widget		text;
	Widget		toggle;
	Widget		workarea;
	extern void	make_frame(char *name, Widget form, Widget *frame_box,
				Widget *frame);
/*
** Bootstrap box
*/
	make_frame("Bootstrap", form, &frame_box, &frame);

	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,      XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,   XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,     XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,    XmATTACH_FORM); ac++;

	workarea = XmCreateForm(frame, "form", al, ac);
	XtManageChild(workarea);
/*
** Radio box
*/
	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,      XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,   XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,     XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,    XmATTACH_NONE); ac++;

	radio_box = XmCreateRadioBox(workarea, "radiobox", al, ac);
	XtManageChild(radio_box);
/*
** IMAKE
*/
	label_string = XmStringCreateSimple("imake args");

	ac = 0;
	XtSetArg(al[ac], XmNlabelString,	label_string); ac++;

	toggle = XmCreateToggleButton(radio_box, "toggle", al, ac);
	XtManageChild(toggle);

	XmStringFree(label_string);

	gui->bootstrap_imake = toggle;
/*
** XMKMF
*/
	label_string = XmStringCreateSimple("xmkmf");

	ac = 0;
	XtSetArg(al[ac], XmNlabelString,	label_string); ac++;

	toggle = XmCreateToggleButton(radio_box, "toggle", al, ac);
	XtManageChild(toggle);

	XmStringFree(label_string);

	gui->bootstrap_xmkmf = toggle;
/*
** CUSTOM
*/
	label_string = XmStringCreateSimple("Custom");

	ac = 0;
	XtSetArg(al[ac], XmNlabelString,	label_string); ac++;

	toggle = XmCreateToggleButton(radio_box, "toggle", al, ac);
	XtManageChild(toggle);

	XmStringFree(label_string);

	gui->bootstrap_custom = toggle;
/*
** Rowcol to hold respective text
*/
	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,      XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,   XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,     XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNleftWidget,     	radio_box); ac++;
	XtSetArg(al[ac], XmNrightAttachment,    XmATTACH_FORM); ac++;

	rowcol = XmCreateRowColumn(workarea, "rowcol", al, ac);
	XtManageChild(rowcol);
/*
** IMAKE text
*/
	ac = 0;

	text = XmCreateText(rowcol, "textfield", al, ac);
	XtManageChild(text);

	gui->bootstrap_imake_text = text;
/*
** XMKMF text
*/
	ac = 0;

	text = XmCreateText(rowcol, "textfield", al, ac);
	XtManageChild(text);

	gui->bootstrap_xmkmf_text = text;
/*
** CUSTOM text
*/
	ac = 0;

	text = XmCreateText(rowcol, "textfield", al, ac);
	XtManageChild(text);

	gui->bootstrap_custom_text = text;
/*
** Return boxed widget
*/
	return(frame_box);
}


/*-----------------------------------------------------------------
** Function:
**	pref_config_box
**
*/

Widget pref_config_box(ib_prefs_gui *gui, Widget form)
{
	int		ac;
	Arg		al[30];
	Widget		frame;
	Widget		frame_box;
	Widget		text;
	extern void	make_frame(char *name, Widget form, Widget *fb,
				Widget *f);
/*
** Config box
*/
	make_frame("Imake Configuration Dir", form, &frame_box, &frame);
/*
** Just one big text field
*/
	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,      XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,   XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,     XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,    XmATTACH_FORM); ac++;

	text = XmCreateText(frame, "textfield", al, ac);
	XtManageChild(text);

	gui->config_location = text;

	return(frame_box);
}

