/*-------------------------------------------------------------------------
**
**                    Copyright (c) 1994 James R. Dyvig
** 
** Permission to use, copy, modify, and distribute this software and its
** documentation for educational, research, and non-profit purposes
** without fee is hereby granted, provided that the above copyright notice
** appear in all copies and that both that copyright notice and this
** permission notice appear in supporting documentation.
** 
** Permission to incorporate this software into commercial products can be
** obtained with the written consent of the author.
** 
** This software is provided AS IS with no warranties of any kind.  The
** author shall have no liability with respect to the infringement of
** copyrights, trade secrets or any patents by this file or any part
** thereof.  In no event will the author be liable for any lost revenue or
** profits or other special, indirect and consequential damages.
**
**-------------------------------------------------------------------------
*/
#include <stdio.h>
#include <Xm/Text.h>
#include "list.h"
#include "utils.h"
#include "ib_target.h"
#include "ib_directory.h"
#include "ib_project.h"
#include "ib_prefs.h"

#define MAXLIST	200

/*-----------------------------------------------------------------
** Function:
**	new_ib_project
**
*/

ib_project *new_ib_project(Widget workArea)
{
	ib_project	*p;
	ib_directory	*d;

	p = (ib_project *) malloc(sizeof(ib_project));

	p->saved = False;
	p->dir_list = make_list(0);
	d = new_ib_directory();
	add(p->dir_list, d);

	strcpy(p->dir_top,".");
	strcpy(p->name,"Unknown");
	p->gui = new_ib_project_gui(workArea);
	p->prefs = new_ib_prefs(workArea);

	p->comments_text = NULL;
	p->custom_text = NULL;

	set_ib_project_gui(p);

	return(p);
}

/*-----------------------------------------------------------------
** Function:
**	zap_ib_project
**
*/

void zap_ib_project(ib_project *p)
{
	ib_directory	*entry;

	entry = (ib_directory *) first(p->dir_list);
	while (entry != NULL) {
		zap_ib_directory(entry);
		entry = (ib_directory *) next(p->dir_list);
	}

	zap_ib_project_gui(p->gui);
	free(p);
}

/*-----------------------------------------------------------------
** Function:
**	set_ib_project_gui
**
*/

void set_ib_project_gui(ib_project *p)
{
	if (p->gui->top_level != NULL) return;
}

/*-----------------------------------------------------------------
** Function:
**	write_imakefile
**
*/

void write_imakefile(ib_project *project)
{
	int		count;
	Boolean		depend_flag = False;
	ib_directory	*directory;
	FILE		*fp;
	Boolean		has_directory = False;
	int		i;
	int		j;
	char		install_dir[MAXCHAR];
	Boolean		lint_flag = False;
	char		man_dir[MAXCHAR];
	ib_prefs	*prefs;
	ib_target	*target;
	int		target_num;
	char		tempname[MAXCHAR];
	char		value[MAXCHAR];
	char		*suffix_ptr;
	char		buffer[80];

	extern void	warning(Widget w, char *info_string);
/*
** Make sure enough info is around to make an Imakefile
*/
	if (project == NULL) return;

	directory = (ib_directory *) first(project->dir_list);
	prefs = project->prefs;
/*
** First, make a backup of the file
*/
	system("cp Imakefile Imakefile.bak");
/*
** Open the file
*/
	fp = fopen("Imakefile","w");
	if (!fp) {
		warning(project->gui->top_level, "Could not create Imakefile");
		return;
	}
/*
** Header info
*/
	fprintf(fp,"#ifndef XCOMM\n");
	fprintf(fp,"#define XCOMM #\n");
	fprintf(fp,"#endif\n");
	fprintf(fp, "XCOMM---------------------------------");
		fprintf(fp, "----------------------------------------\n");
	fprintf(fp,
"XCOMM               Imakefile built using Imake Builder %s\n", IB_VERSION);
	fprintf(fp,
"XCOMM                         (C) Copyright %s\n", IB_COPYRIGHT_YEAR);

	fprintf(fp,"XCOMM\n");
	fprintf(fp,"XCOMM To generate the Makefile:\n");
	fprintf(fp,
"XCOMM    imake -DBuildMotif -DUseInstalledMotif -DUseInstalled \\\n");
	fprintf(fp,
"XCOMM                                            -I/usr/lib/X11/config\n");

	if (prefs->compiler == IB_CC) {
		fprintf(fp,"XCOMM\n");
		fprintf(fp,"XCOMM Supplying debug flags:\n");
		fprintf(fp,"XCOMM    make \"CDEBUGFLAGS = -g\"\n");
	}

	if (project->comments_text != NULL &&
		strcmp(project->comments_text,"") !=0) {
		fprintf(fp, "XCOMM---------------------------------");
		fprintf(fp, "----------------------------------------\n");

		fprintf(fp, "XCOMM ");

		for (i=0;i<strlen(project->comments_text);i++) {
			if (project->comments_text[i] == '\n') {
				fprintf(fp, "\nXCOMM ");
			} else {
				fprintf(fp, "%c",
					project->comments_text[i]);
			}
		}

		fprintf(fp, "\n");
	}

	fprintf(fp, "XCOMM---------------------------------");
		fprintf(fp, "----------------------------------------\n");
/*
** Up front info
*/
	fprintf(fp, "\n");

	fprintf(fp, "#ifdef UltrixArchitecture\n");
	fprintf(fp, "XMLIB = -lXm\n");
	fprintf(fp, "#endif\n");

	if (! prefs->use_motif_1_2) {
		fprintf(fp, "\n");
		fprintf(fp, "#ifdef SGIArchitecture\n");
		fprintf(fp, "SYSLIBS = -lPW\n");
		fprintf(fp, "#endif\n");
	}

	fprintf(fp, "\n");
/*
** Custom options (if any)
*/
	if (project->custom_text != NULL &&
		strcmp(project->custom_text,"") !=0) {

		fprintf(fp, "XCOMM----- CUSTOM OPTIONS ------------------\n");

		for (i=0;i<strlen(project->custom_text);i++) {
			fprintf(fp, "%c", project->custom_text[i]);
		}

		fprintf(fp, "\n");

		fprintf(fp, "XCOMM----- CUSTOM OPTIONS (END) ------------\n");

		fprintf(fp, "\n");
	}
/*
** Subdirectories
*/
	target = (ib_target *) first(directory->target_list);
	while (target != NULL) {
		if (target->type == IB_DIRECTORY) {
			has_directory = True;
			break;
		}

		target = (ib_target *) next(directory->target_list);
	}

	if (has_directory) {
		fprintf(fp, "#define IHaveSubdirs\n");
		fprintf(fp, "\n");
	}
/*
** Foreach target:
*/
	target_num = 1;
	target = (ib_target *) first(directory->target_list);

	if (target == NULL) {
		warning(project->gui->top_level,
			"No Targets Defined");
	}

	while (target != NULL) {
/*
** Header
*/
		fprintf(fp, "XCOMM --------------------------------------\n");
		fprintf(fp, "XCOMM Target:  %s\n", target->name);

		if (target->comments_text != NULL &&
			strcmp(target->comments_text,"") !=0) {

			fprintf(fp, "XCOMM\nXCOMM ");
			for (i=0;i<strlen(target->comments_text);i++) {
				if (target->comments_text[i] == '\n') {
					fprintf(fp, "\nXCOMM ");
				} else {
					fprintf(fp, "%c",
						target->comments_text[i]);
				}
			}

			fprintf(fp,"\n");
		}

		fprintf(fp, "XCOMM --------------------------------------\n");
		fprintf(fp, "\n");
/*
** Treate directories differently
*/
		if (target->type == IB_DIRECTORY) {
			if (target->source_count < 1) {
				warning(project->gui->top_level,
					"No Dirs.  Use <OK> on Target Panel");
			}
	
			if (target->source_count > 0) {
				fprintf(fp, "SUBDIRS = \\\n");
				for (i=0; i<target->source_count - 1;i++) {
					fprintf(fp, "\t%s \\\n",
						target->source_list[i]);
				}
				fprintf(fp, "\t%s\n", target->source_list[i]);
			} else {
				fprintf(fp, "SUBDIRS = \n");
			}

			fprintf(fp, "\n");
		} else {
/*
** Libraries
*/
			fprintf(fp, "INCLUDE%d = %s\n", target_num,
				target->include_dirs);

			fprintf(fp, "LOCAL_LIBS%d = %s\n", target_num,
				target->local_libs);

			if (target->use_f77_lib)
				fprintf(fp,
					"F77LIBS = -lF77 -lU77 -lI77 -lisam\n");

			if (target->use_gl_lib)
				fprintf(fp, "GLXLIBS = -lfm_s -lgl_s\n");

			fprintf(fp, "SYS_LIBS%d = ", target_num);
			if (target->use_f77_lib) fprintf(fp, "$(F77LIBS) ");
			if (target->use_gl_lib) fprintf(fp, "$(GLXLIBS) ");
			if (target->use_xm_lib) fprintf(fp, "$(XMLIB) ");
			if (target->use_xaw_lib) fprintf(fp, "$(XAWLIB) ");
			if (target->use_xt_lib) fprintf(fp, "$(XTOOLLIB) ");
			if (target->use_x_lib) fprintf(fp, "$(XLIB) ");
			if (target->use_xm_lib)
				fprintf(fp, "$(SYSLIBS) $(XMEXTRA_LIBS) ");
			fprintf(fp, "-lm\n");

			fprintf(fp, "\n");
/*
** Deps
*/
			if (target->use_depend &&
				strcmp(target->depend_text, "") != 0) {
				fprintf(fp, "DEPS%d = %s\n", target_num,
					target->depend_text);
			} else {
				fprintf(fp, "DEPS%d = \n", target_num);
			}
/*
** Sources
*/
			if (target->source_count < 1) {
				warning(project->gui->top_level,
					"No Files.  Use <OK> on Target Panel");
			}

			if (target->source_count > 0) {
				fprintf(fp, "SRCS%d = \\\n", target_num);
				for (i=0; i<target->source_count - 1;i++) {
					fprintf(fp, "\t%s \\\n",
						target->source_list[i]);
				}
				fprintf(fp, "\t%s\n", target->source_list[i]);
			} else {
				fprintf(fp, "SRCS%d = \n", target_num);
			}

			fprintf(fp, "\n");
/*
** Objects
*/
			if (target->source_count > 0) {
				fprintf(fp, "OBJS%d = \\\n", target_num);
				for (i=0; i<target->source_count - 1;i++) {
					strcpy(tempname,target->source_list[i]);
					for (j=strlen(tempname)-1;j>=0;j--) {
						if (tempname[j] == '.') {
							suffix_ptr =
								&tempname[j+1];
							break;
						}
					}

					if (j > 0)
						strcpy(suffix_ptr, "o");
					else {
						sprintf(buffer,
							"Invalid file: %s",
							tempname);

						warning(project->gui->top_level,
						buffer);
					}

					fprintf(fp, "\t%s \\\n", tempname);
				}
				strcpy(tempname, target->source_list[i]);
				for (j=strlen(tempname)-1;j>=0;j--) {
					if (tempname[j] == '.') {
						suffix_ptr = &tempname[j+1];
						break;
					}
				}

				if (j > 0)
					strcpy(suffix_ptr, "o");
				else {
					sprintf(buffer,
						"Invalid file: %s",
						tempname);

					warning(project->gui->top_level,
						buffer);
				}

				fprintf(fp, "\t%s\n", tempname);
			} else {
				fprintf(fp, "SRCS%d = \n", target_num);
			}
	
			fprintf(fp, "\n");
		}
/*
** Program rules
*/
		switch (target->type) {
		case IB_PROGRAM:
			fprintf(fp, "AllTarget(%s)\n", target->name);

			fprintf(fp, "NormalProgramTarget(%s,", target->name);
			fprintf(fp, "$(OBJS%d),",target_num);
			fprintf(fp, "$(DEPS%d),",target_num);
			fprintf(fp, "$(LOCAL_LIBS%d),", target_num);
			fprintf(fp, "$(SYS_LIBS%d))\n", target_num);

			if (target->use_install) {
				if (strcmp(target->install_text, "") == 0) {
					strcpy(install_dir, "NullParameter");
				} else {
					strcpy(install_dir,
						target->install_text);
				}
	
				fprintf(fp, "InstallProgram(%s,%s)\n",
					target->name, install_dir);
			}
			break;
		case IB_LIBRARY:
			fprintf(fp, "NormalLibraryObjectRule()\n");
			fprintf(fp, "NormalLibraryTarget(%s,",target->name);
			fprintf(fp, "$(OBJS%d))\n", target_num);

			if (target->use_install) {
				if (strcmp(target->install_text, "") == 0) {
					strcpy(install_dir, "NullParameter");
				} else {
					strcpy(install_dir,
						target->install_text);
				}
	
				fprintf(fp, "InstallLibrary(%s,%s)\n",
					target->name, install_dir);
			}
			break;
		case IB_DIRECTORY:
			break;
		}
/*
** Man Page
*/
		if (target->use_man) {
			if (strcmp(target->man_text, "") != 0) {
				strcpy(man_dir, target->install_text);

				fprintf(fp, "InstallManPage(%s,%s)\n",
					target->name, man_dir);
			}
		}

		fprintf(fp, "\n");

		if (target->use_depend) depend_flag = True;
		if (target->use_lint) lint_flag = True;

		target_num++;
		target = (ib_target *) next(directory->target_list);
	}
/*
** Ending statements
*/
	fprintf(fp, "XCOMM --------------------------------------\n");
	fprintf(fp, "XCOMM The finishing touches...\n");
	fprintf(fp, "XCOMM --------------------------------------\n");
	fprintf(fp, "\n");
/*
** Compilers
*/
	switch (prefs->compiler) {
	case IB_CC:
		/* Nothing to do...use default compiler */
		break;
	case IB_GCC:
		fprintf(fp, "CC = gcc\n\n");
		break;
	case IB_GCPP:
		fprintf(fp, "CC = g++\n\n");
		break;
	case IB_CUSTOM_CC:
		fprintf(fp, "CC = %s\n\n", prefs->compiler_text);
		break;
	}
/*
** Includes
*/
	fprintf(fp, "SYS_INCLUDES = %s\n\n", prefs->sys_includes);

	count = 1;
	fprintf(fp, "INCLUDES = ");
	target = (ib_target *) first(directory->target_list);
	while (target != NULL) {
		fprintf(fp, "$(INCLUDE%d) ", count);

		target = (ib_target *) next(directory->target_list);
		count++;
	}

	fprintf(fp, "$(SYS_INCLUDES)\n");
/*
** Flags
*/
	if (prefs->compiler == IB_CC) {
		fprintf(fp, "CCOPTIONS = $(INCLUDES) %s\n\n", prefs->cflags);
	} else if (prefs->compiler == IB_GCC) {
		fprintf(fp, "CCOPTIONS = $(INCLUDES) %s\n", prefs->cflags);
		fprintf(fp, "CDEBUGFLAGS = \n\n");
	} else if (prefs->compiler == IB_GCPP) {
		fprintf(fp, "CPPFLAGS = $(INCLUDES) %s\n", prefs->cflags);
		fprintf(fp, "CCOPTIONS = $(INCLUDES) %s\n", prefs->cflags);
		fprintf(fp, "CDEBUGFLAGS = \n\n");
	} else {
		fprintf(fp, "CCOPTIONS = $(INCLUDES) %s\n", prefs->cflags);
		fprintf(fp, "CDEBUGFLAGS = \n\n");
	}

	if (strcmp(prefs->fflags, "") != 0)
		fprintf(fp, "FFLAGS = %s\n\n", prefs->fflags);
	if (strcmp(prefs->customflags, "") != 0) {
		fprintf(fp, "XCOMM Custom Flags\n");
		fprintf(fp, "%s\n\n", prefs->customflags);
	}

	fprintf(fp, "\n");
/*
** For all targets...
**
** SRCS
*/
	target_num = 1;
	target = (ib_target *) first(directory->target_list);

	fprintf(fp, "SRCS = ");

	while (target != NULL) {
		fprintf(fp, "$(SRCS%d) ",target_num);
		target_num++;
		target = (ib_target *) next(directory->target_list);
	}

	fprintf(fp, "\n");
/*
** OBJS
*/
	target_num = 1;
	target = (ib_target *) first(directory->target_list);

	fprintf(fp, "OBJS = ");

	while (target != NULL) {
		fprintf(fp, "$(OBJS%d) ",target_num);
		target_num++;
		target = (ib_target *) next(directory->target_list);
	}

	fprintf(fp, "\n\n");
/*
** If subdirs
*/
	if (has_directory) {
		fprintf(fp, "MakeSubdirs($(SUBDIRS))\n");
		fprintf(fp, "DependSubdirs($(SUBDIRS))\n");
	}
/*
** Depend libs
*/
	if (depend_flag) {
		fprintf(fp, "DependTarget()\n");
	}
/*
** Lint
*/
	if (lint_flag) {
		fprintf(fp, "LintTarget()\n");
	}
/*
** Close and return
*/
	fclose(fp);

	project->saved = True;
}

/*-----------------------------------------------------------------
** Function:
**	load_imakefile
**
*/

void load_imakefile(ib_project *project)
{
	char		buffer[200];
	int		count = 0;
	Boolean		depend_flag = False;
	ib_directory	*directory;
	FILE		*fp;
	Boolean		has_directory = False;
	int		i;
	char		install_dir[MAXCHAR];
	int		is_ib_file = False;
	char		keyword1[40];
	char		keyword2[40];
	char		keyword3[40];
	Boolean		lint_flag = False;
	ib_prefs	*prefs;
	ib_target	*target;
	int		target_num;
	char		tempname[MAXCHAR];
	char		value[MAXCHAR];
	Boolean		odd_header = False;
	int		num_text_lines;
	int		total_length;
	char		*text_line[MAXLIST];
	char		*ptr;
	int		comment_offset;
	int		num_markers = 0;
	extern void	warning(Widget w, char *info_string);
	extern void	parse_target_input(char *buffer, FILE *fp,
				ib_project *project);
/*
** Make sure enough info is around to make an Imakefile
*/
	if (project == NULL) return;

	directory = (ib_directory *) first(project->dir_list);
	prefs = project->prefs;
/*
** Open the file
*/
	fp = fopen("Imakefile","r");
	if (!fp) {
		return;
	}
/*
** Verify Imakefile was built by IB
*/
	while (fgets(buffer, 200, fp)) {
		if (strstr(buffer,"Imake Builder") != NULL) {
			is_ib_file = True;
			break;
		}
	}

	if (is_ib_file)
		rewind(fp);
	else {
		fclose(fp);
		warning(project->gui->top_level, "Not an IB Imakefile");
		return;
	}
/*
** Now, read in any comments
*/
	num_text_lines = 0;
	total_length = 0;
/*
** Look for second marker "-----------"
** Amazingly enought, first check of XCOMM/# gets us by the
** #ifdef XCOMM trio at the top of the file...how lucky can
** we get?
*/
	while (fgets(buffer, 200, fp)) {
		if (strncmp(buffer,"XCOMM",5) != 0 &&
			strncmp(buffer,"#",1) != 0) {
			odd_header = True;
			break;
		}
		if (strncmp(buffer, "XCOMM---",8) == 0 ||
			strncmp(buffer, "#---",4) == 0) {
			num_markers++;
			if (num_markers == 2) {
				if (strncmp(buffer, "#---",4) == 0)
					comment_offset = 2;
				else
					comment_offset = 6;
				break;
			}
		}
	}

	if (! odd_header) {
		while (fgets(buffer, 200, fp)) {
			if (strncmp(buffer,"XCOMM",5) != 0 &&
				strncmp(buffer,"#",1) != 0) break;
			if (strncmp(buffer, "XCOMM---",8) == 0 ||
				strncmp(buffer, "#---",4) == 0) break;

			ptr = &buffer[comment_offset];

			text_line[num_text_lines] = 
				(char *) malloc(sizeof(char)*strlen(ptr)+1);
			strcpy(text_line[num_text_lines], ptr);

			total_length += strlen(ptr);
			num_text_lines++;
		}

		if (num_text_lines > 0) {
			project->comments_text = (char *) malloc(sizeof(char) *
				total_length + 1);

			strcpy(project->comments_text, text_line[0]);
			free(text_line[0]);

			for (i=1;i<num_text_lines;i++) {
				strcat(project->comments_text, text_line[i]);
				free(text_line[i]);
			}

			project->comments_text[
				strlen(project->comments_text)-1] = '\000';
		}
	}
/*
** Read line by line, looking for keywords
*/
	while (fgets(buffer, 200, fp)) {
		count = sscanf(buffer, "%s %s", keyword1, keyword2);
		if (count > 0) {
			if (strcmp(keyword2, "SGIArchitecture") == 0) {
				prefs->use_motif_1_2 = False;
			} else if (strcmp(keyword2, "CUSTOM") == 0) {
				num_text_lines = 0;
				total_length = 0;

				while (fgets(buffer, 200, fp)) {
					sscanf(buffer, "%s %s",
						keyword1, keyword2);

					if (strcmp(keyword2, "CUSTOM") == 0) {
						break;
					}

					ptr = buffer;

					text_line[num_text_lines] = 
						(char *) malloc(sizeof(char)*
						strlen(ptr)+1);

					strcpy(text_line[num_text_lines], ptr);

					total_length += strlen(ptr);
					num_text_lines++;
				}

				if (num_text_lines > 0) {
					project->custom_text =
						(char *) malloc(sizeof(char) *
						total_length + 1);

					strcpy(project->custom_text,
						text_line[0]);
					free(text_line[0]);

					for (i=1;i<num_text_lines;i++) {
						strcat(project->custom_text,
							text_line[i]);
						free(text_line[i]);
					}

					project->custom_text[
						strlen(project->custom_text)-1]
						= '\000';
				}
			} else if (strcmp(keyword1, "CC") == 0) {
				sscanf(buffer, "%s %s %s", keyword1,
					keyword2, keyword3);
				if (strcmp(keyword3, "cc") == 0) {
					prefs->compiler = IB_CC;
				} else if (strcmp(keyword3, "gcc") == 0) {
					prefs->compiler = IB_GCC;
				} else if (strcmp(keyword3, "g++") == 0) {
					prefs->compiler = IB_GCPP;
				} else {
					prefs->compiler = IB_CUSTOM_CC;
					sscanf(buffer,
				   "%s %s %[a-z,A-Z, ,-,.,/,0-9,_,=,+,',\"]",
					   keyword1, keyword2,
					   prefs->compiler_text);
				}
			} else if (strcmp(keyword1, "CFLAGS") == 0) {
				sscanf(buffer,
			   "%s %s %s %[a-z,A-Z, ,-,.,/,0-9,_,=,+,',\"]",
					keyword1, keyword2, keyword3,
					prefs->cflags);
			} else if (strcmp(keyword1, "CCOPTIONS") == 0) {
				sscanf(buffer,
			   "%s %s %s %[a-z,A-Z, ,-,.,/,0-9,_,=,+,',\"]",
					keyword1, keyword2, keyword3,
					prefs->cflags);
			} else if (strcmp(keyword1, "FFLAGS") == 0) {
				sscanf(buffer,
			   "%s %s %[a-z,A-Z, ,-,.,/,0-9,_,=,+,',\"]",
					keyword1, keyword2,
					prefs->fflags);
			} else if (strcmp(keyword2, "Custom") == 0) {
				fgets(buffer, 200, fp);
				sscanf(buffer,
			   "%[a-z,A-Z, ,-,.,/,0-9,_,=,+,',\"]",
					prefs->customflags);
			} else if (strcmp(keyword1, "SYS_INCLUDES") == 0) {
				sscanf(buffer,
			   "%s %s %[a-z,A-Z, ,-,.,/,0-9,_,=,+,',\"]",
					keyword1, keyword2,
					prefs->sys_includes);
			} else if (strcmp(keyword1, "XCOMM") == 0 ||
				strcmp(keyword1, "#") == 0) {
				if (strcmp(keyword2, "Target:") == 0) {
					parse_target_input(buffer, fp, project);
				}
			} else if (strcmp(keyword1, "DependTarget") == 0) {
			} else if (strncmp(keyword1, "LintTarget",10) == 0) {
				target = (ib_target *)
					first(directory->target_list);
				while (target != NULL) {
					target->use_lint = True;
					set_ib_target_gui(target);
					target = (ib_target *)
						next(directory->target_list);
				}
			}
		}
	}
/*
** Close and return
*/
	fclose(fp);

	project->saved = False;
}

/*-----------------------------------------------------------------
** Function:
**	parse_target_input
**
*/

void parse_target_input(char *buffer, FILE *fp, ib_project *project)
{
	char		local_buf[200];
	int		count = 0;
	Boolean		depend_flag = False;
	ib_directory	*directory;
	int		i;
	char		install_dir[MAXCHAR];
	Boolean		has_directory = False;
	char		keyword1[40];
	char		keyword2[40];
	char		keyword3[40];
	Boolean		lint_flag = False;
	int		num;
	ib_prefs	*prefs;
	ib_target	*target;
	int		target_num;
	char		target_name[80];
	char		tempname[MAXCHAR];
	char		value[MAXCHAR];
	ib_project_gui	*gui;
	XmString	item_string;
	int		*pos_list;
	char		*ptr;
	int		comment_offset;
	int		status;
	char		slist[MAXLIST][80];
	int		num_text_lines;
	int		total_length;
	char		*text_line[MAXLIST];

	extern void	warning(Widget w, char *info_string);
	extern void	set_dir_sensitivity_cb(Widget w, ib_target_gui *t,
				XmToggleButtonCallbackStruct *cbs);
/*
** Make sure enough info is around to make an Imakefile
*/
	if (project == NULL) return;

	directory = (ib_directory *) first(project->dir_list);
	prefs = project->prefs;
/*
** Get the target name
*/
	count = sscanf(buffer, "%s %s %s", keyword1, keyword2, target_name);
	if (count != 3) {
		warning(project->gui->top_level,"Problem loading Imakefile");
		return;
	}
/*
** Create a new target and name it
*/
	gui = project->gui;
/*
** Add to list
*/
	item_string = XmStringCreateSimple(target_name);

	if (! XmListGetMatchPos(gui->target_list, item_string,
		&pos_list, &count)) {
		XmListAddItem(gui->target_list, item_string, 0);
/*
** Create a new target node
*/
		target = new_ib_target(gui->top_level,target_name);

		directory = (ib_directory *) first(project->dir_list);
		add(directory->target_list, target);
	}

	XmStringFree(item_string);
/*
** Now, read in any comments
*/
	num_text_lines=0;
	total_length = 0;

	while (fgets(local_buf, 200, fp)) {
		if (strncmp(local_buf,"XCOMM",5) != 0 &&
			strncmp(local_buf,"#",1) != 0) break;
		if (strncmp(local_buf, "XCOMM --",8) == 0 ||
			strncmp(local_buf, "# --",4) == 0) break;
		
		if (strncmp(local_buf, "#", 1) == 0) {
			comment_offset = 2;
		} else {
			comment_offset = 6;
		}

		ptr = &buffer[comment_offset];

		text_line[num_text_lines] = 
			(char *) malloc(sizeof(char)*strlen(ptr)+1);
		strcpy(text_line[num_text_lines], ptr);

		total_length += strlen(ptr);
		num_text_lines++;
	}

	if (num_text_lines > 0) {
		target->comments_text = (char *) malloc(sizeof(char) *
			total_length + 1);

		strcpy(target->comments_text, text_line[0]);
		free(text_line[0]);

		for (i=1;i<num_text_lines;i++) {
			strcat(target->comments_text, text_line[i]);
			free(text_line[i]);
		}

		target->comments_text[strlen(target->comments_text)-1] = '\000';
	}
/*
** Read line by line, looking for keywords
*/
	while (fgets(local_buf, 200, fp)) {
		count = sscanf(local_buf, "%s %s %s",
			keyword1, keyword2, keyword3);
		if (count > 0) {
/*
** SUBDIRS
*/
			if (strcmp(keyword1, "SUBDIRS") == 0) {
				num = 0;
				target->type = IB_DIRECTORY;

				set_dir_sensitivity_cb(NULL, target->gui, NULL);

				if (strcmp(keyword3,"\\") == 0) {
					while (fgets(local_buf, 200, fp)) {
						count = sscanf(local_buf,
							"%s %s",
							keyword1, keyword2);
						if (count > 0) {
							strcpy(slist[num++],
								keyword1);
							if (num == MAXLIST) {
							   fprintf(stderr,
							     "Too many dirs\n");
							   exit(-1);
							}
						}

						if (count == 1) {
							break;
						}
					}
				}

				if (num > 0) {
					 target->source_list = (char **)
					 	malloc(sizeof (char *) * num);
				} else {
					 target->source_list = (char **)
					 	malloc(sizeof (char *) * 1);
				}

				for (i=0;i<num;i++) {
					target->source_list[i] = (char *)
						malloc(sizeof(char) *
						(strlen(slist[i])+1));
					strcpy(target->source_list[i],slist[i]);
					target->source_count++;
				}
/*
** INCLUDE
*/
			} else if (strncmp(keyword1, "INCLUDE",7) == 0) {
				sscanf(local_buf,
				   "%s %s %[a-z,A-Z, ,-,.,/,0-9,_,=,+,',\"]",
					keyword1, keyword2,
					target->include_dirs);
/*
** LOCLAL LIBS
*/
			} else if (strncmp(keyword1, "LOCAL_LIBS",10) == 0) {
				sscanf(local_buf,
				   "%s %s %[a-z,A-Z, ,-,.,/,0-9,_,=,+,',\"]",
					keyword1, keyword2,
					target->local_libs);
/*
** F77 LIBS
*/
			} else if (strcmp(keyword1, "F77LIBS") == 0) {
				target->use_f77_lib = True;
/*
** GLX LIBS
*/
			} else if (strcmp(keyword1, "GLXLIBS") == 0) {
				target->use_gl_lib = True;
/*
** SYS LIBS
*/
			} else if (strncmp(keyword1, "SYS_LIBS",8) == 0) {
				if (strstr(local_buf, "-lXm") != NULL ||
					strstr(local_buf, "XMLIB") != NULL) {
					target->use_xm_lib = True;
				} else {
					target->use_xm_lib = False;
				}

				if (strstr(local_buf, "XAWLIB") != NULL) {
					target->use_xaw_lib = True;
				} else {
					target->use_xaw_lib = False;
				}

				if (strstr(local_buf, "XTOOLLIB") != NULL) {
					target->use_xt_lib = True;
				} else {
					target->use_xt_lib = False;
				}

				if (strstr(local_buf, "XLIB") != NULL) {
					target->use_x_lib = True;
				} else {
					target->use_x_lib = False;
				}
/*
** Dependencies
*/
			} else if (strncmp(keyword1, "DEPS",4) == 0) {
				count = sscanf(local_buf,
				   "%s %s %[a-z,A-Z, ,-,.,/,0-9,_,=,+,',\"]",
					keyword1, keyword2,
					target->depend_text);
				if (count != 3) {
					strcpy(target->depend_text, "");
				}
				target->use_depend = True;
/*
** SRCS
*/
			} else if (strncmp(keyword1, "SRCS",4) == 0) {
				num = 0;

				if (strcmp(keyword3,"\\") == 0) {
					while (fgets(local_buf, 200, fp)) {
						count = sscanf(local_buf,
							"%s %s",
							keyword1, keyword2);
						if (count > 0) {
							strcpy(slist[num++],
								keyword1);
							if (num == MAXLIST) {
							   fprintf(stderr,
							     "Too many srcs\n");
							   exit(-1);
							}
						}

						if (count == 1) {
							break;
						}
					}
				}

				if (num > 0) {
					 target->source_list = (char **)
					 	malloc(sizeof (char *) * num);
				} else {
					 target->source_list = (char **)
					 	malloc(sizeof (char *) * 1);
				}

				for (i=0;i<num;i++) {
					target->source_list[i] = (char *)
						malloc(sizeof(char) *
						(strlen(slist[i])+1));
					strcpy(target->source_list[i],slist[i]);
					target->source_count++;
				}
/*
** Program type
*/
			} else if (strncmp(keyword1, "NormalProgram",13) == 0) {
				target->type = IB_PROGRAM;
/*
** Library type
*/
			} else if (strncmp(keyword1, "NormalLibrary",13) == 0) {
				target->type = IB_LIBRARY;
/*
** Install dir
*/
			} else if (strncmp(keyword1, "InstallPro",10) == 0 ||
				strncmp(keyword1, "InstallLib",10) == 0) {
				ptr = strstr(local_buf, ",");
				if (ptr != NULL) {
					ptr++;
					i = 0;
					while (ptr[i] != ')') {
						target->install_text[i] =
							ptr[i];
						i++;
					}
					target->install_text[i] = '\000';
				}

				target->use_install = True;
/*
** Install Man Page dir
*/
			} else if (strncmp(keyword1, "InstallMan",10) == 0) {
				ptr = strstr(local_buf, ",");
				if (ptr != NULL) {
					ptr++;
					i = 0;
					while (ptr[i] != ')') {
						target->man_text[i] =
							ptr[i];
						i++;
					}
					target->man_text[i] = '\000';
				}

				target->use_man = True;
/*
** New Section
*/
			} else if (strcmp(keyword1, "XCOMM") == 0 ||
				strcmp(keyword1, "#") == 0) {
				if (strcmp(keyword2, "Target:") == 0) {
					parse_target_input(local_buf,
						fp, project);
					break;
				} else if (strcmp(keyword2, "The") == 0) {
					break;
				}
			}
		}
	}

	set_ib_target_gui(target);
}

/*-----------------------------------------------------------------
** Function:
**	ib_edit_header
**
*/

void ib_edit_header(Widget w, ib_project *project)
{
	int		ac;
	Arg		al[30];
	char		buffer[200];
	char		*contents;
	Widget		form;
	Widget		scrolled_text;
	extern void	edit_headerCB(Widget, XtPointer, XtPointer);
/*
** Open a new window and place the text in it
*/
	sprintf(buffer,"HEADER COMMENTS");
	form = new_edit_window(w, "Imake Builder", buffer, 600, 300,
		(XtCallbackProc) edit_headerCB, (XtPointer) project);

	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNeditMode,		XmMULTI_LINE_EDIT); ac++;
	XtSetArg(al[ac], XmNeditable,		True); ac++;

	scrolled_text = XmCreateScrolledText(form, "textblock", al, ac);
	XtManageChild(scrolled_text);

	project->gui->comments_text = scrolled_text;

	XmTextSetString(scrolled_text, project->comments_text);
}

/*-----------------------------------------------------------------
** Function:
**	ib_edit_custom_options
**
*/

void ib_edit_custom_options(Widget w, ib_project *project)
{
	int		ac;
	Arg		al[30];
	char		buffer[200];
	char		*contents;
	Widget		form;
	Widget		scrolled_text;
	extern void	edit_custom_optionsCB(Widget, XtPointer, XtPointer);
/*
** Open a new window and place the text in it
*/
	sprintf(buffer,"CUSTOM OPTIONS");
	form = new_edit_window(w, "Imake Builder", buffer, 600, 300,
		(XtCallbackProc) edit_custom_optionsCB, (XtPointer) project);

	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNeditMode,		XmMULTI_LINE_EDIT); ac++;
	XtSetArg(al[ac], XmNeditable,		True); ac++;

	scrolled_text = XmCreateScrolledText(form, "textblock", al, ac);
	XtManageChild(scrolled_text);

	project->gui->custom_text = scrolled_text;

	XmTextSetString(scrolled_text, project->custom_text);
}

