/*-------------------------------------------------------------------------
**
**                    Copyright (c) 1994 James R. Dyvig
** 
** Permission to use, copy, modify, and distribute this software and its
** documentation for educational, research, and non-profit purposes
** without fee is hereby granted, provided that the above copyright notice
** appear in all copies and that both that copyright notice and this
** permission notice appear in supporting documentation.
** 
** Permission to incorporate this software into commercial products can be
** obtained with the written consent of the author.
** 
** This software is provided AS IS with no warranties of any kind.  The
** author shall have no liability with respect to the infringement of
** copyrights, trade secrets or any patents by this file or any part
** thereof.  In no event will the author be liable for any lost revenue or
** profits or other special, indirect and consequential damages.
**
**-------------------------------------------------------------------------
*/
#include <Xm/List.h>
#include <Xm/Text.h>
#include <Xm/MessageB.h>

#include "ib_project.h"
#include "ib_target.h"
#include "utils.h"

/*-----------------------------------------------------------------
** Function:
**	add_project_cb
**
*/

void add_project_cb(Widget w, XtPointer data, XtPointer cbs)
{
	int		count;
	ib_directory	*directory;
	ib_project_gui	*gui;
	XmString	item_string;
	int		*pos_list;
	int		status;
	ib_target	*target;
	char		target_name[80];
	char		*text;
	extern ib_project *project;

	gui = (ib_project_gui *) data;

	text = XmTextGetString(gui->target);

	status = sscanf(text, "%s", target_name);
	if (status != 1) return;
/*
** Add to list
*/
	item_string = XmStringCreateSimple(target_name);

	if (! XmListGetMatchPos(gui->target_list, item_string,
		&pos_list, &count)) {
		XmListAddItem(gui->target_list, item_string, 0);
/*
** Create a new target node
*/
		target = new_ib_target(w,target_name);

		directory = (ib_directory *) first(project->dir_list);
		add(directory->target_list, target);
	}

	XmStringFree(item_string);

	project->saved = False;

	return;
}

/*-----------------------------------------------------------------
** Function:
**	delete_project_cb
**
*/

void project_del_target_cb(Widget w, XtPointer data, XtPointer cbs)
{
	int		count;
	ib_project_gui	*gui;
	XmString	item_string;
	int		*pos_list;
	int		status;
	char		target_name[80];
	char		*text;
	extern ib_project *project;

	gui = (ib_project_gui *) data;

	text = XmTextGetString(gui->target);

	status = sscanf(text, "%s", target_name);
	if (status != 1) return;
/*
** Delete from list
*/
	item_string = XmStringCreateSimple(target_name);

	if (XmListGetMatchPos(gui->target_list, item_string,
		&pos_list, &count)) {

		XmListDeleteItem(gui->target_list, item_string);
		zap_named_ib_target(target_name);
	}

	XmStringFree(item_string);
/*
** Reset the current target input line
*/
	XmTextSetString(gui->target, "");

	project->saved = False;

	return;
}

void delete_project_cb(Widget w, XtPointer data, XtPointer cbs)
{
	Widget		dialog;
	XmString	msg, yes, no;
	Widget		toplevel;

	toplevel = w;
	while (toplevel && !XtIsTopLevelShell(toplevel))
		toplevel = XtParent(toplevel);

	dialog = XmCreateQuestionDialog(toplevel, "dialog", NULL, 0);
	yes = XmStringCreateSimple("Yes");
	no = XmStringCreateSimple("No");
	msg = XmStringCreateSimple("Really Delete Target?");

	XtVaSetValues(dialog,
		XmNmessageString,	msg,
		XmNokLabelString,	yes,
		XmNcancelLabelString,	no,
		XmNdialogStyle, 	XmDIALOG_FULL_APPLICATION_MODAL,
		NULL);
	
	XtAddCallback(dialog, XmNokCallback, project_del_target_cb, data);
	XtAddCallback(dialog, XmNcancelCallback,
		(XtCallbackProc) XtDestroyWidget, NULL);

	XmStringFree(yes);
	XmStringFree(no);
	XmStringFree(msg);

	XtManageChild(dialog);
	XtPopup(XtParent(dialog), XtGrabNone);

	return;
}

/*-----------------------------------------------------------------
** Function:
**	set_current_target_cb
**
*/

void set_current_target_cb(Widget w, XtPointer data, XmListCallbackStruct *cbs)
{
	ib_project_gui	*gui;
	int		status;
	ib_target	*target;
	char		target_name[80];
	char		*text;

	gui = (ib_project_gui *) data;

	XmStringGetLtoR(cbs->item, XmSTRING_DEFAULT_CHARSET, &text);

	status = sscanf(text, "%s", target_name);
	if (status != 1) return;
/*
** Set current target name
*/
	XmTextSetString(gui->target, target_name);

	XtFree(text);
/*
** Show the gui if it is not up
*/
	target = get_named_ib_target(target_name);
	if (target != NULL) {
		set_ib_target_gui(target);
		show_ib_target_gui(target->gui);
	}

	return;
}

/*-----------------------------------------------------------------
** Function:
**	edit_headerCB
**
*/

void edit_headerCB(Widget w, XtPointer data, XtPointer cbs)
{
	ib_project	*project;
	Widget		topLevel;

	project = (ib_project *) data;

	topLevel = w;
	while (topLevel && !XtIsTopLevelShell(topLevel))
		topLevel = XtParent(topLevel);


	project->comments_text = XmTextGetString(project->gui->comments_text);

	del_window(topLevel, topLevel, NULL);
}

/*-----------------------------------------------------------------
** Function:
**	edit_custom_optionsCB
**
*/

void edit_custom_optionsCB(Widget w, XtPointer data, XtPointer cbs)
{
	ib_project	*project;
	Widget		topLevel;

	project = (ib_project *) data;

	topLevel = w;
	while (topLevel && !XtIsTopLevelShell(topLevel))
		topLevel = XtParent(topLevel);


	project->custom_text = XmTextGetString(project->gui->custom_text);

	del_window(topLevel, topLevel, NULL);
}
