/*-------------------------------------------------------------------------
**
**                    Copyright (c) 1994 James R. Dyvig
** 
** Permission to use, copy, modify, and distribute this software and its
** documentation for educational, research, and non-profit purposes
** without fee is hereby granted, provided that the above copyright notice
** appear in all copies and that both that copyright notice and this
** permission notice appear in supporting documentation.
** 
** Permission to incorporate this software into commercial products can be
** obtained with the written consent of the author.
** 
** This software is provided AS IS with no warranties of any kind.  The
** author shall have no liability with respect to the infringement of
** copyrights, trade secrets or any patents by this file or any part
** thereof.  In no event will the author be liable for any lost revenue or
** profits or other special, indirect and consequential damages.
**
**-------------------------------------------------------------------------
*/
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/List.h>
#include <Xm/PushBG.h>
#include <Xm/Separator.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>

#include "ib_project.h"

/*-----------------------------------------------------------------
** Function:
**	new_ib_project_gui
**
*/

ib_project_gui *new_ib_project_gui(Widget workArea)
{
	ib_project_gui	*p;

	p = (ib_project_gui *) malloc(sizeof(ib_project_gui));

	p->target_list = NULL;
	p->target = NULL;
	p->top_level = NULL;
	p->comments_text = NULL;

	create_ib_project_gui(p, workArea);

	return(p);
}

/*-----------------------------------------------------------------
** Function:
**	show_ib_project_gui
**
*/

void show_ib_project_gui(ib_project_gui *p)
{
	if (p->top_level == NULL) {
		return;
	} else {
		XtRealizeWidget(p->top_level);
	}
}

/*-----------------------------------------------------------------
** Function:
**	dismiss_ib_project_gui
**
*/

void dimiss_ib_project_gui(ib_project_gui *p)
{
	if (p->top_level == NULL) {
		return;
	} else {
		XtUnrealizeWidget(p->top_level);
	}
}

/*-----------------------------------------------------------------
** Function:
**	zap_ib_project_gui
**
*/

void zap_ib_project_gui(ib_project_gui *p)
{
	if (p->top_level == NULL) {
		return;
	} else {
		XtDestroyWidget(p->top_level);
		p->top_level = NULL;
	}
}

/*-----------------------------------------------------------------
** Function:
**	create_ib_project_gui
**
*/

void create_ib_project_gui(ib_project_gui *p, Widget workArea)
{
	int		ac = 0;
	Arg		al[30];
	Widget		button;
	Widget		form;
	Widget		frame;
	Widget		frame_box;
	Widget		label;
	XmString	label_string;
	Widget		separator;
	Widget		target_list;
	Widget		text;
	Widget		workarea;
	extern void add_project_cb(Widget w, XtPointer data, XtPointer cbs);
	extern void delete_project_cb(Widget w, XtPointer data, XtPointer cbs);
	extern void set_current_target_cb(Widget w, XtPointer data,
		XtPointer cbs);
	extern void	make_frame(char *name, Widget form, Widget *frame_box,
		Widget *frame);

	if (p->top_level != NULL) return;
/*
** Create the background frame
*/
	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNmarginWidth,	5); ac++;
	XtSetArg(al[ac], XmNmarginHeight,	5); ac++;

	frame = XmCreateFrame(workArea, "frame", al, ac);
	XtManageChild(frame);
/*
** Create the background form
*/
	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_FORM); ac++;

	form = XmCreateForm(frame, "form", al, ac);
	XtManageChild(form);
/*
** Add current target input text field
*/
	make_frame("Current Target", form, &frame_box, &workarea);

	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_FORM); ac++;

	XtSetValues(frame_box, al, ac);
		
	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_FORM); ac++;

	text = XmCreateText(workarea, "textfield", al, ac);
	XtManageChild(text);

	XtAddCallback(text, XmNactivateCallback, add_project_cb,
		(XtPointer) p);
/*
** Add separator
*/
	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNtopWidget,		frame_box); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_FORM); ac++;

	separator = XmCreateSeparator(form, "separator", al, ac);
	XtManageChild(separator);
/*
** Create the target list frame
*/
	make_frame("Target List", form, &frame_box, &workarea);

	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNtopWidget,		separator); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_FORM); ac++;

	XtSetValues(frame_box, al, ac);
/*
** Create the background form
*/
	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_FORM); ac++;

	form = XmCreateForm(workarea, "form", al, ac);
	XtManageChild(form);
/*
** Create Add button
*/
	label_string = XmStringCreateSimple("Add Target");

	ac = 0;
	XtSetArg(al[ac], XmNbottomAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNlabelString,	label_string); ac++;

	button = XmCreatePushButtonGadget(form, "button", al, ac);
	XtManageChild(button);

	XmStringFree(label_string);

	XtAddCallback(button, XmNactivateCallback, add_project_cb,
		(XtPointer) p);
/*
** Create Delete button
*/
	label_string = XmStringCreateSimple("Delete Target");

	ac = 0;
	XtSetArg(al[ac], XmNbottomAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNlabelString,	label_string); ac++;

	button = XmCreatePushButtonGadget(form, "button", al, ac);
	XtManageChild(button);

	XmStringFree(label_string);

	XtAddCallback(button, XmNactivateCallback, delete_project_cb,
		(XtPointer) p);
/*
** Add target list with label
*/
	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,	XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNbottomWidget,	button); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNselectionPolicy,	XmSINGLE_SELECT); ac++;

	target_list = XmCreateScrolledList(form, "list", al, ac);
	XtManageChild(target_list);

	XtAddCallback(target_list, XmNsingleSelectionCallback,
		set_current_target_cb, (XtPointer) p);
/*
** Set data structs
*/
	p->top_level = frame;
	p->target_list = target_list;
	p->target = text;
}
