/*-------------------------------------------------------------------------
**
**                    Copyright (c) 1994 James R. Dyvig
** 
** Permission to use, copy, modify, and distribute this software and its
** documentation for educational, research, and non-profit purposes
** without fee is hereby granted, provided that the above copyright notice
** appear in all copies and that both that copyright notice and this
** permission notice appear in supporting documentation.
** 
** Permission to incorporate this software into commercial products can be
** obtained with the written consent of the author.
** 
** This software is provided AS IS with no warranties of any kind.  The
** author shall have no liability with respect to the infringement of
** copyrights, trade secrets or any patents by this file or any part
** thereof.  In no event will the author be liable for any lost revenue or
** profits or other special, indirect and consequential damages.
**
**-------------------------------------------------------------------------
*/
#include <Xm/Xm.h>
#include "list.h"
#include "ib_target.h"
#include "ib_directory.h"
#include "ib_project.h"

/*-----------------------------------------------------------------
** Function:
**	new_ib_target
**
*/

ib_target *new_ib_target(Widget workArea, char *name)
{
	ib_target	*t;

	t = (ib_target *) malloc(sizeof(ib_target));

	t->file_list = make_list(0);
	strcpy(t->name, name);
	strcpy(t->include_dirs,"");
	strcpy(t->local_libs,"");
	strcpy(t->depend_text,"");
	strcpy(t->install_text,"$(BINDIR)");
	strcpy(t->man_text,"$(MAN)");
	t->gui = new_ib_target_gui(workArea, name);
	t->type = IB_PROGRAM;
	t->use_depend = False;
	t->use_f77_lib = False;
	t->use_gl_lib = False;
	t->use_install = False;
	t->use_lint = False;
	t->use_man = False;
	t->use_x_lib = True;
	t->use_xt_lib = True;
	t->use_xaw_lib = False;
	t->use_xm_lib = True;
	t->source_count = 0;
	t->source_list = NULL;
	t->comments_text = NULL;

	set_ib_target_gui(t);

	return(t);
}

/*-----------------------------------------------------------------
** Function:
**	zap_ib_target
**
*/

void zap_ib_target(ib_target *t)
{
	ib_directory		*directory;
	ib_target		*target;
	extern ib_project	*project;

	zap_ib_target_gui(t->gui);

	directory = (ib_directory *) first(project->dir_list);
	target = (ib_target *) first(directory->target_list);
	while (target != NULL) {
		if (target == t) {
			delete(directory->target_list);
			break;
		}
		target = (ib_target *) next(directory->target_list);
	}

	free(t);
}

/*-----------------------------------------------------------------
** Function:
**	get_named_ib_target
**
*/

ib_target *get_named_ib_target(char *name)
{
	ib_directory		*directory;
	ib_target		*target;
	extern ib_project	*project;

	directory = (ib_directory *) first(project->dir_list);
	target = (ib_target *) first(directory->target_list);
	while (target != NULL) {
		if (strcmp(target->name, name) == 0) {
			return(target);
			break;
		}
		target = (ib_target *) next(directory->target_list);
	}

	return(NULL);
}

/*-----------------------------------------------------------------
** Function:
**	zap_named_ib_target
**
*/

void zap_named_ib_target(char *name)
{
	ib_target		*target;

	target = get_named_ib_target(name);
	if (target != NULL) zap_ib_target(target);
}
