/*-------------------------------------------------------------------------
**
**                    Copyright (c) 1994 James R. Dyvig
** 
** Permission to use, copy, modify, and distribute this software and its
** documentation for educational, research, and non-profit purposes
** without fee is hereby granted, provided that the above copyright notice
** appear in all copies and that both that copyright notice and this
** permission notice appear in supporting documentation.
** 
** Permission to incorporate this software into commercial products can be
** obtained with the written consent of the author.
** 
** This software is provided AS IS with no warranties of any kind.  The
** author shall have no liability with respect to the infringement of
** copyrights, trade secrets or any patents by this file or any part
** thereof.  In no event will the author be liable for any lost revenue or
** profits or other special, indirect and consequential damages.
**
**-------------------------------------------------------------------------
*/
#include <Xm/FileSB.h>
#include <Xm/List.h>
#include <Xm/Text.h>
#include <Xm/MessageB.h>

#include "ib_project.h"
#include "ib_target.h"

/*-----------------------------------------------------------------
** Function:
**	target_del_target_cb
**
*/

void del_for_good_cb(Widget w, ib_target_gui *target, XtPointer cbs)
{
	int			count;
	ib_project_gui		*gui;
	XmString		item_string;
	int			*pos_list;
	int			status;
	char			target_name[80];
	char			*text;
	extern ib_project	*project;

	gui = project->gui;
/*
** Delete from list
*/
	item_string = XmStringCreateSimple(target->target_name);

	if (XmListGetMatchPos(gui->target_list, item_string,
		&pos_list, &count)) {

		XmListDeleteItem(gui->target_list, item_string);
		zap_named_ib_target(target->target_name);
	}

	XmStringFree(item_string);
/*
** Reset the current target input line
*/
	XmTextSetString(gui->target, "");

	project->saved = False;

	return;
}

void target_del_target_cb(Widget w, ib_target_gui *target, XtPointer cbs)
{
	Widget		dialog;
	XmString	msg, yes, no;
	Widget		toplevel;

	toplevel = w;
	while (toplevel && !XtIsTopLevelShell(toplevel))
		toplevel = XtParent(toplevel);

	dialog = XmCreateQuestionDialog(toplevel, "dialog", NULL, 0);
	yes = XmStringCreateSimple("Yes");
	no = XmStringCreateSimple("No");
	msg = XmStringCreateSimple("Really Delete Target?");

	XtVaSetValues(dialog,
		XmNmessageString,	msg,
		XmNokLabelString,	yes,
		XmNcancelLabelString,	no,
		XmNdialogStyle, 	XmDIALOG_FULL_APPLICATION_MODAL,
		NULL);
	
	XtAddCallback(dialog, XmNokCallback,
		(XtCallbackProc) del_for_good_cb, (XtPointer) target);
	XtAddCallback(dialog, XmNcancelCallback, 
		(XtCallbackProc) XtDestroyWidget, NULL);

	XmStringFree(yes);
	XmStringFree(no);
	XmStringFree(msg);

	XtManageChild(dialog);
	XtPopup(XtParent(dialog), XtGrabNone);

	return;
}

/*-----------------------------------------------------------------
** Function:
**	target_dismiss_cb
**
*/

void target_dismiss_cb(Widget w, ib_target_gui *gui, XtPointer cbs)
{
	ib_target	*target;
	extern ib_project *project;

	dimiss_ib_target_gui(gui);

	target = get_named_ib_target(gui->target_name);
	if (target != NULL) {
		get_ib_target_gui(target);
		project->saved = False;
	}

	return;
}

/*-----------------------------------------------------------------
** Function:
**	target_cancel_cb
**
*/

void target_cancel_cb(Widget w, ib_target_gui *gui, XtPointer cbs)
{
	ib_target	*target;

	dimiss_ib_target_gui(gui);

	return;
}

/*-----------------------------------------------------------------
** Function:
**	target_addlist_cb
**
*/

void target_addlist_cb(Widget w, ib_target_gui *target, XtPointer cbs)
{
	if (target->file_select_dialog == NULL) return;

	XtManageChild(target->file_select_dialog);

	return;
}

/*-----------------------------------------------------------------
** Function:
**	target_dellist_cb
**
*/

void target_dellist_cb(Widget w, ib_target_gui *target, XtPointer cbs)
{
	int	*pos_list;
	int	count;
	Boolean	status;
	int	i;

	if (XmListGetSelectedPos(target->file_list, &pos_list, &count)) {
		for (i=count-1;i>=0;i--) {
			XmListDeletePos(target->file_list, pos_list[i]);
		}
	}
	return;
}

/*-----------------------------------------------------------------
** Function:
**	target_resetlist_cb
**
*/

void target_resetlist_cb(Widget w, ib_target_gui *target, XtPointer cbs)
{
	XmString	*items;
	XmString	*new_items;
	char		*new_s;
	char		*s;
	int		ac;
	Arg		al[30];
	int		count = -1;
	Widget		dialog;
	int		i;
	XmString 	item_string;
	XmString	new_name;
	extern char	*strip_path(char *c);
	extern ib_project *project;

	if (target->file_select_dialog == NULL) return;

	XmListDeleteAllItems(target->file_list);

	dialog = target->file_select_dialog;

	item_string = XmStringCreateSimple(project->prefs->file_match);
	XmFileSelectionDoSearch(dialog, item_string);
	XmStringFree(item_string);

	if (! XmToggleButtonGetState(target->is_directory)) {
		XtVaGetValues(dialog,
			XmNfileListItemCount,	&count,
			XmNfileListItems,	&items,
			NULL);

		new_items = (XmString *) XtMalloc(sizeof(XmString)*count);
		for (i=0;i<count;i++) {
			XmStringGetLtoR(items[i], XmSTRING_DEFAULT_CHARSET, &s);
			new_s = strip_path(s);
			new_items[i] = XmStringCreateSimple(new_s);
			XtFree(s);
		}

		XmListAddItems(target->file_list, new_items, count, 0);
		for (i=0;i<count;i++) XmStringFree(new_items[i]);
	} else {
		XtVaGetValues(dialog,
			XmNdirListItemCount,	&count,
			XmNdirListItems,	&items,
			NULL);
		new_items = (XmString *) XtMalloc(sizeof(XmString)*count);
		for (i=0;i<count;i++) {
			XmStringGetLtoR(items[i], XmSTRING_DEFAULT_CHARSET, &s);
			new_s = strip_path(s);
			new_items[i] = XmStringCreateSimple(new_s);
			if (strcmp(new_s, ".") != 0 &&
				strcmp(new_s, "..") != 0 &&
				strcmp(new_s, "RCS") != 0) {
				XmListAddItem(target->file_list,
					new_items[i],0);
			}
			XtFree(s);
		}
	
		for (i=0;i<count;i++) XmStringFree(new_items[i]);
	}

	return;
}


/*-----------------------------------------------------------------
** Function:
**	target_file_sel_ok_cb
**
*/

void target_file_sel_ok_cb(Widget w, ib_target_gui *target,
	XmFileSelectionBoxCallbackStruct *cbs)
{
	char		*file;
	XmString	item;
	char		*new_file;
	int		count;
	int		*pos_list;
	extern char	*strip_path(char *s);

	XmStringGetLtoR(cbs->value, XmSTRING_DEFAULT_CHARSET, &file);
	new_file = strip_path(file);

	item = XmStringCreateSimple(new_file);
	if (! XmListGetMatchPos(target->file_list, item, &pos_list, &count)) {
		XmListAddItem(target->file_list, item, 0);
	}

	XtFree(file);
	XmStringFree(item);

	return;
}

/*-----------------------------------------------------------------
** Function:
**	target_file_sel_dismiss_cb
**
*/

void target_file_sel_dismiss_cb(Widget w, ib_target_gui *target, XtPointer cbs)
{
	XtUnmanageChild(target->file_select_dialog);
}

/*-----------------------------------------------------------------
** Function:
**	set_dir_sensitivity_cb
**
*/

void set_dir_sensitivity_cb(Widget w, ib_target_gui *t,
	XmToggleButtonCallbackStruct *cbs)
{
	Widget		dialog_file_list;
	extern void	target_resetlist_cb(Widget w, ib_target_gui *target,
				XtPointer cbs);

	dialog_file_list = XmFileSelectionBoxGetChild(t->file_select_dialog,
		XmDIALOG_LIST);
	XtSetSensitive(dialog_file_list, False);

	XtSetSensitive(t->includes, False);
	XtSetSensitive(t->local_libs, False);
	XtSetSensitive(t->use_depend, False);
	XtSetSensitive(t->depend_text, False);
	XtSetSensitive(t->use_install, False);
	XtSetSensitive(t->install_text, False);
	XtSetSensitive(t->use_lint, False);
	XtSetSensitive(t->use_man, False);
	XtSetSensitive(t->man_text, False);
	XtSetSensitive(t->use_x_lib, False);
	XtSetSensitive(t->use_xaw_lib, False);
	XtSetSensitive(t->use_xm_lib, False);
	XtSetSensitive(t->use_xt_lib, False);
	XtSetSensitive(t->use_f77_lib, False);
	XtSetSensitive(t->use_gl_lib, False);

	target_resetlist_cb(w, t, NULL);
}

/*-----------------------------------------------------------------
** Function:
**	set_program_sensitivity_cb
**
*/

void set_program_sensitivity_cb(Widget w, ib_target_gui *t,
	XmToggleButtonCallbackStruct *cbs)
{
	Widget		dialog_file_list;
	extern void	target_resetlist_cb(Widget w, ib_target_gui *target,
				XtPointer cbs);

	dialog_file_list = XmFileSelectionBoxGetChild(t->file_select_dialog,
		XmDIALOG_LIST);
	XtSetSensitive(dialog_file_list, True);

	XtSetSensitive(t->includes, True);
	XtSetSensitive(t->local_libs, True);
	XtSetSensitive(t->use_depend, True);
	XtSetSensitive(t->depend_text, True);
	XtSetSensitive(t->use_install, True);
	XtSetSensitive(t->install_text, True);
	XtSetSensitive(t->use_lint, True);
	XtSetSensitive(t->use_man, True);
	XtSetSensitive(t->man_text, True);
	XtSetSensitive(t->use_x_lib, True);
	XtSetSensitive(t->use_xaw_lib, True);
	XtSetSensitive(t->use_xm_lib, True);
	XtSetSensitive(t->use_xt_lib, True);
	XtSetSensitive(t->use_f77_lib, True);
	XtSetSensitive(t->use_gl_lib, True);

	target_resetlist_cb(w, t, NULL);
}
