/*-------------------------------------------------------------------------
**
**                    Copyright (c) 1994 James R. Dyvig
** 
** Permission to use, copy, modify, and distribute this software and its
** documentation for educational, research, and non-profit purposes
** without fee is hereby granted, provided that the above copyright notice
** appear in all copies and that both that copyright notice and this
** permission notice appear in supporting documentation.
** 
** Permission to incorporate this software into commercial products can be
** obtained with the written consent of the author.
** 
** This software is provided AS IS with no warranties of any kind.  The
** author shall have no liability with respect to the infringement of
** copyrights, trade secrets or any patents by this file or any part
** thereof.  In no event will the author be liable for any lost revenue or
** profits or other special, indirect and consequential damages.
**
**-------------------------------------------------------------------------
*/
#include <stdio.h>
#include <Xm/FileSB.h>
#include <Xm/BulletinB.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/List.h>
#include <Xm/PushBG.h>
#include <Xm/RowColumn.h>
#include <Xm/Separator.h>
#include <Xm/Text.h>
#include <Xm/ToggleB.h>

#include "ib_target.h"
#include "ib_prefs.h"
#include "ib_project.h"
#include "utils.h"

/*-----------------------------------------------------------------
** Function:
**	new_ib_target_gui
**
*/

ib_target_gui *new_ib_target_gui(Widget workArea, char *name)
{
	ib_target_gui	*p;

	p = (ib_target_gui *) malloc(sizeof(ib_target_gui));

	strcpy(p->target_name, name);
	p->file_list = NULL;
	p->file_select_dialog = NULL;
	p->includes = NULL;
	p->is_directory = NULL;
	p->is_library = NULL;
	p->is_program = NULL;
	p->local_libs = NULL;
	p->name = NULL;
	p->top_level = NULL;
	p->use_depend = NULL;
	p->depend_text = NULL;
	p->use_install = NULL;
	p->install_text = NULL;
	p->use_lint = NULL;
	p->use_man = NULL;
	p->man_text = NULL;
	p->comments_text = NULL;
	p->use_x_lib = NULL;
	p->use_xaw_lib = NULL;
	p->use_xm_lib = NULL;
	p->use_xt_lib = NULL;
	p->use_f77_lib = NULL;
	p->use_gl_lib = NULL;

	create_ib_target_gui(p, workArea);

	return(p);
}

/*-----------------------------------------------------------------
** Function:
**	show_ib_target_gui
**
*/

void show_ib_target_gui(ib_target_gui *p)
{
	if (p->top_level == NULL) {
		return;
	} else {
		XtRealizeWidget(p->top_level);
	}
}

/*-----------------------------------------------------------------
** Function:
**	dismiss_ib_target_gui
**
*/

void dimiss_ib_target_gui(ib_target_gui *p)
{
	if (p->top_level == NULL) {
		return;
	} else {
		XtUnrealizeWidget(p->top_level);
	}
}

/*-----------------------------------------------------------------
** Function:
**	zap_ib_target_gui
**
*/

void zap_ib_target_gui(ib_target_gui *p)
{
	if (p->top_level == NULL) {
		return;
	} else {
		XtUnrealizeWidget(p->top_level);
		XtDestroyWidget(p->top_level);
		p->top_level = NULL;
	}
}

/*-----------------------------------------------------------------
** Function:
**	strip_path
**
*/

char *strip_path(char *s)
{
	int	i;
	int	len;

	len = strlen(s);
	for (i=len-2;i>0;i--) {
		if (s[i] == '/') return &s[i+1];
	}

	return(s);
}

/*-----------------------------------------------------------------
** Function:
**	get_source_files
**
*/

void get_source_files(ib_target_gui *gui, Widget w, Widget source_list)
{
	Widget		button;
	XmString	*items;
	XmString	*new_items;
	char		*new_s;
	char		*s;
	int		ac;
	Arg		al[30];
	int		count = -1;
	Widget		dialog;
	int		i;
	XmString 	item_string;
	XmString	new_name;
	extern ib_project *project;
	extern void	target_file_sel_ok_cb(Widget w, ib_target_gui *gui,
				XmFileSelectionBoxCallbackStruct *cbs);
	extern void	target_file_sel_dismiss_cb(Widget w,ib_target_gui *gui,
				XtPointer cbs);

	if (gui->file_select_dialog != NULL) return;

	ac = 0;
	dialog = XmCreateFileSelectionDialog(w, "Source", al, ac);

	button = XmFileSelectionBoxGetChild(dialog, XmDIALOG_HELP_BUTTON);
	XtSetSensitive(button, False);

	button = XmFileSelectionBoxGetChild(dialog, XmDIALOG_OK_BUTTON);
	new_name = XmStringCreateSimple("Add");
	XtVaSetValues(button, XmNlabelString, new_name, NULL);
	XmStringFree(new_name);

	button = XmFileSelectionBoxGetChild(dialog, XmDIALOG_CANCEL_BUTTON);
	new_name = XmStringCreateSimple("Dismiss");
	XtVaSetValues(button, XmNlabelString, new_name, NULL);
	XmStringFree(new_name);

	XtAddCallback(dialog, XmNokCallback,
		(XtCallbackProc) target_file_sel_ok_cb, (XtPointer) gui);

	XtAddCallback(dialog, XmNcancelCallback,
		(XtCallbackProc) target_file_sel_dismiss_cb, (XtPointer) gui);

	item_string = XmStringCreateSimple(project->prefs->file_match);
	XmFileSelectionDoSearch(dialog, item_string);
	XmStringFree(item_string);

	XtVaGetValues(dialog,
		XmNfileListItemCount,	&count,
		XmNfileListItems,	&items,
		NULL);

	new_items = (XmString *) XtMalloc(sizeof(XmString)*count);
	for (i=0;i<count;i++) {
		XmStringGetLtoR(items[i], XmSTRING_DEFAULT_CHARSET, &s);
		new_s = strip_path(s);
		new_items[i] = XmStringCreateSimple(new_s);
		XtFree(s);
	}

	XmListAddItems(source_list, new_items, count, 0);
	for (i=0;i<count;i++) XmStringFree(new_items[i]);

	gui->file_select_dialog = dialog;
}

/*-----------------------------------------------------------------
** Function:
**	create_ib_target_gui
**
*/

void create_ib_target_gui(ib_target_gui *p, Widget workArea)
{
	int		ac;
	Arg		al[30];
	Widget		bottom_separator;
	Widget		button;
	Widget		comment_box;
	Widget		comment_separator;
	Widget		form;
	Widget		frame;
	Widget		include_box;
	Widget		label;
	XmString	label_string;
	Widget		lib_box;
	Widget		option_box;
	Widget		rowcol;
	Widget		shell;
	Widget		source_box;
	Widget		top_level;
	Widget		top_separator;
	Widget		type_box;
	extern void	target_del_target_cb(Widget w, ib_target_gui *target,
		XtPointer cbs);
	extern void	target_dismiss_cb(Widget w, ib_target_gui *target,
		XtPointer cbs);
	extern void	target_cancel_cb(Widget w, ib_target_gui *target,
		XtPointer cbs);
	extern Widget	target_include_box(ib_target_gui *gui, Widget form);
	extern Widget	target_lib_box(ib_target_gui *gui, Widget form);
	extern Widget	target_option_box(ib_target_gui *gui, Widget form);
	extern Widget	target_source_box(ib_target_gui *gui, Widget form);
	extern Widget	target_type_box(ib_target_gui *gui, Widget form);
	extern Widget	target_comment_box(ib_target_gui *gui, Widget form);

	if (p->top_level != NULL) return;

	top_level = workArea;
	while (top_level && ! XtIsTopLevelShell(top_level))
		top_level = XtParent(top_level);

	shell = XtVaAppCreateShell("Target", "IB",
		topLevelShellWidgetClass, XtDisplay(top_level),
		NULL, 0);
	
/*
** Create a form to work from
*/
	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,      XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,   XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,     XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,    XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNmarginWidth,	4); ac++;
	XtSetArg(al[ac], XmNmarginHeight,       4); ac++;
	XtSetArg(al[ac], XmNfractionBase,       2); ac++;

	form = XmCreateForm(shell, "form", al, ac);
	XtManageChild(form);
/*
** Label
*/
	label_string = XmStringCreateSimple(p->target_name);

	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,      XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,     XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,    XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNalignment,  	XmALIGNMENT_CENTER); ac++;
	XtSetArg(al[ac], XmNlabelString,  	label_string); ac++;

	label = XmCreateLabel(form, "titlelabel", al, ac);
	XtManageChild(label);

	XmStringFree(label_string);
/*
** Separator
*/
	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,      XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNtopWidget,	  	label); ac++;
	XtSetArg(al[ac], XmNleftAttachment,     XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,    XmATTACH_FORM); ac++;

	top_separator = XmCreateSeparator(form, "separator", al, ac);
	XtManageChild(top_separator);
/*
** Rowcol for delete and dismiss
*/
	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,      XmATTACH_NONE); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,   XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,     XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,    XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNorientation,    	XmHORIZONTAL); ac++;
	XtSetArg(al[ac], XmNadjustLast,    	False); ac++;

	rowcol = XmCreateRowColumn(form, "rowcol", al, ac);
	XtManageChild(rowcol);

	ac = 0;
	button = XmCreatePushButtonGadget(rowcol, "OK", al, ac);
	XtManageChild(button);

	XtAddCallback(button, XmNactivateCallback,
		(XtCallbackProc) target_dismiss_cb, (XtPointer) p);

	button = XmCreatePushButtonGadget(rowcol, "Cancel", al, ac);
	XtManageChild(button);

	XtAddCallback(button, XmNactivateCallback,
		(XtCallbackProc) target_cancel_cb, (XtPointer) p);

	button = XmCreatePushButtonGadget(rowcol, "Delete", al, ac);
	XtManageChild(button);

	XtAddCallback(button, XmNactivateCallback,
		(XtCallbackProc) target_del_target_cb, (XtPointer) p);
/*
** Separator
*/
	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,      XmATTACH_NONE); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,   XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNbottomWidget,   	rowcol); ac++;
	XtSetArg(al[ac], XmNleftAttachment,     XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,    XmATTACH_FORM); ac++;

	bottom_separator = XmCreateSeparator(form, "separator", al, ac);
	XtManageChild(bottom_separator);
/*
** Comment box
*/
	comment_box = target_comment_box(p, form);

	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,      XmATTACH_NONE); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,	XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNbottomWidget,	bottom_separator); ac++;
	XtSetArg(al[ac], XmNleftAttachment,     XmATTACH_NONE); ac++;
	XtSetArg(al[ac], XmNrightAttachment,    XmATTACH_FORM); ac++;

	XtSetValues(comment_box, al, ac);
/*
** Separator
*/
	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,      XmATTACH_NONE); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,   XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNbottomWidget,   	comment_box); ac++;
	XtSetArg(al[ac], XmNleftAttachment,     XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,    XmATTACH_FORM); ac++;

	comment_separator = XmCreateSeparator(form, "separator", al, ac);
	XtManageChild(comment_separator);
/*
** Target lib box
*/
	lib_box = target_lib_box(p, form);

	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,      XmATTACH_NONE); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,	XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNbottomWidget,	comment_separator); ac++;
	XtSetArg(al[ac], XmNleftAttachment,     XmATTACH_NONE); ac++;
	XtSetArg(al[ac], XmNrightAttachment,    XmATTACH_FORM); ac++;

	XtSetValues(lib_box, al, ac);
/*
** Target Include box
*/
	include_box = target_include_box(p, form);

	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,      XmATTACH_NONE); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,	XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNbottomWidget,	lib_box); ac++;
	XtSetArg(al[ac], XmNleftAttachment,     XmATTACH_OPPOSITE_WIDGET); ac++;
	XtSetArg(al[ac], XmNleftWidget,     	lib_box); ac++;
	XtSetArg(al[ac], XmNrightAttachment,    XmATTACH_FORM); ac++;

	XtSetValues(include_box, al, ac);
/*
** Source file box
*/
	source_box = target_source_box(p, form);

	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,      XmATTACH_OPPOSITE_WIDGET); ac++;
	XtSetArg(al[ac], XmNtopWidget,	  	include_box); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,	XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNbottomWidget,	comment_separator); ac++;
	XtSetArg(al[ac], XmNleftAttachment,     XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,    XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNrightWidget,    	lib_box); ac++;

	XtSetValues(source_box, al, ac);
/*
** Target Option box
*/
	option_box = target_option_box(p, form);

	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,      XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNtopWidget,	  	top_separator); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,	XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNbottomWidget,	include_box); ac++;
	XtSetArg(al[ac], XmNleftAttachment,     XmATTACH_OPPOSITE_WIDGET); ac++;
	XtSetArg(al[ac], XmNleftWidget,     	include_box); ac++;
	XtSetArg(al[ac], XmNrightAttachment,    XmATTACH_FORM); ac++;

	XtSetValues(option_box, al, ac);
/*
** Type box
*/
	type_box = target_type_box(p, form);

	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,      XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNtopWidget,	  	top_separator); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,	XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNbottomWidget,	source_box); ac++;
	XtSetArg(al[ac], XmNleftAttachment,     XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,    XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNrightWidget,    	option_box); ac++;

	XtSetValues(type_box, al, ac);
/*
** Clean up and go
*/
	p->top_level = shell;
	p->name = label;

	XtRealizeWidget(shell);
}

/*-----------------------------------------------------------------
** Function:
**	target_comment_box
**
*/

Widget target_comment_box(ib_target_gui *gui, Widget form)
{
	int		ac;
	Arg		al[30];
	Widget		frame;
	Widget		frame_box;
/*
** Type box
*/
	make_frame("Comments", form, &frame_box, &frame);

	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNcolumns,		62); ac++;
	XtSetArg(al[ac], XmNrows,		8); ac++;
	XtSetArg(al[ac], XmNeditMode,		XmMULTI_LINE_EDIT); ac++;

	gui->comments_text = XmCreateText(frame, "textfield", al, ac);
	XtManageChild(gui->comments_text);

	return(frame_box);
}

/*-----------------------------------------------------------------
** Function:
**	target_type_box
**
*/

Widget target_type_box(ib_target_gui *gui, Widget form)
{
	int		ac;
	Arg		al[30];
	Widget		frame;
	Widget		frame_box;
	Widget		label;
	XmString	label_string;
	Widget		radiobox;
	Widget		toggle;
	extern void	set_program_sensitivity_cb(Widget w, ib_target_gui *gui,
				XmToggleButtonCallbackStruct *cbs);
	extern void	set_dir_sensitivity_cb(Widget w, ib_target_gui *gui,
				XmToggleButtonCallbackStruct *cbs);
/*
** Type box
*/
	make_frame("Type", form, &frame_box, &frame);

	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,      XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNtopWidget,	  	label); ac++;
	XtSetArg(al[ac], XmNleftAttachment,     XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,    XmATTACH_FORM); ac++;

	radiobox = XmCreateRadioBox(frame, "radiobox", al, ac);
	XtManageChild(radiobox);
/*
** Program radio button
*/
	label_string = XmStringCreateSimple("Program");

	ac = 0;
	XtSetArg(al[ac], XmNlabelString,	label_string); ac++;

	toggle = XmCreateToggleButton(radiobox, "toggle", al, ac);
	XtManageChild(toggle);

	XmStringFree(label_string);

	XtAddCallback(toggle, XmNdisarmCallback,
		(XtCallbackProc) set_program_sensitivity_cb,
		(XtPointer) gui);

	gui->is_program = toggle;
/*
** Library radio button
*/
	label_string = XmStringCreateSimple("Library");

	ac = 0;
	XtSetArg(al[ac], XmNlabelString,	label_string); ac++;

	toggle = XmCreateToggleButton(radiobox, "toggle", al, ac);
	XtManageChild(toggle);

	XmStringFree(label_string);

	XtAddCallback(toggle, XmNdisarmCallback,
		(XtCallbackProc) set_program_sensitivity_cb,
		(XtPointer) gui);

	gui->is_library = toggle;
/*
** Directory radio button
*/
	label_string = XmStringCreateSimple("Directory");

	ac = 0;
	XtSetArg(al[ac], XmNlabelString,	label_string); ac++;

	toggle = XmCreateToggleButton(radiobox, "toggle", al, ac);
	XtManageChild(toggle);

	XmStringFree(label_string);

	XtAddCallback(toggle, XmNdisarmCallback,
		(XtCallbackProc) set_dir_sensitivity_cb,
		(XtPointer) gui);

	gui->is_directory = toggle;

	return(frame_box);
}

/*-----------------------------------------------------------------
** Function:
**	target_option_box
**
*/

Widget target_option_box(ib_target_gui *gui, Widget form)
{
	int		ac;
	Arg		al[30];
	Widget		checkbox;
	Widget		frame;
	Widget		frame_box;
	Widget		label;
	XmString	label_string;
	Widget		rowcol;
	Widget		text;
	Widget		toggle;
	Widget		workarea;
/*
** Type box
*/
	make_frame("Target Options", form, &frame_box, &frame);

	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,      XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,   XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,     XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,    XmATTACH_FORM); ac++;

	workarea = XmCreateForm(frame, "form", al, ac);
	XtManageChild(workarea);
/*
** Check box
*/
	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,      XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,   XmATTACH_NONE); ac++;
	XtSetArg(al[ac], XmNleftAttachment,     XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,    XmATTACH_NONE); ac++;

	checkbox = XmCreateSimpleCheckBox(workarea, "checkbox", al, ac);
	XtManageChild(checkbox);
/*
** Depend check button
*/
	label_string = XmStringCreateSimple("Dependencies");

	ac = 0;
	XtSetArg(al[ac], XmNlabelString,	label_string); ac++;

	toggle = XmCreateToggleButton(checkbox, "toggle", al, ac);
	XtManageChild(toggle);

	XmStringFree(label_string);

	gui->use_depend = toggle;
/*
** Install check button
*/
	label_string = XmStringCreateSimple("Install Dir");

	ac = 0;
	XtSetArg(al[ac], XmNlabelString,	label_string); ac++;

	toggle = XmCreateToggleButton(checkbox, "toggle", al, ac);
	XtManageChild(toggle);

	XmStringFree(label_string);

	gui->use_install = toggle;
/*
** Lint check button
*/
	label_string = XmStringCreateSimple("Lint");

	ac = 0;
	XtSetArg(al[ac], XmNlabelString,	label_string); ac++;

	toggle = XmCreateToggleButton(checkbox, "toggle", al, ac);
	XtManageChild(toggle);

	XmStringFree(label_string);

	gui->use_lint = toggle;
/*
** Manual check button
*/
	label_string = XmStringCreateSimple("Man Page");

	ac = 0;
	XtSetArg(al[ac], XmNlabelString,	label_string); ac++;

	toggle = XmCreateToggleButton(checkbox, "toggle", al, ac);
	XtManageChild(toggle);

	XmStringFree(label_string);

	gui->use_man = toggle;
/*
** Add input text for some of the toggles
*/
	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,      XmATTACH_OPPOSITE_WIDGET); ac++;
	XtSetArg(al[ac], XmNtopWidget,      	checkbox); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,   XmATTACH_OPPOSITE_WIDGET); ac++;
	XtSetArg(al[ac], XmNbottomWidget,      	checkbox); ac++;
	XtSetArg(al[ac], XmNleftAttachment,     XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNleftWidget,     	checkbox); ac++;
	XtSetArg(al[ac], XmNrightAttachment,    XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNnumColumns,    	1); ac++;
	XtSetArg(al[ac], XmNpacking,    	XmPACK_COLUMN); ac++;

	rowcol = XmCreateRowColumn(workarea, "rowcol", al, ac);
	XtManageChild(rowcol);

	ac = 0;

	text = XmCreateText(rowcol, "textfield", al, ac);
	XtManageChild(text);
	gui->depend_text = text;

	text = XmCreateText(rowcol, "textfield", al, ac);
	XtManageChild(text);
	gui->install_text = text;

	text = XmCreateText(rowcol, "textfield", al, ac);
	XtManageChild(text);
	XmTextSetString(text, "<No Args>");
	XtSetSensitive(text, False);

	text = XmCreateText(rowcol, "textfield", al, ac);
	XtManageChild(text);
	gui->man_text = text;
/*
** Return boxed widget
*/
	return(frame_box);
}

/*-----------------------------------------------------------------
** Function:
**	target_include_box
**
*/

Widget target_include_box(ib_target_gui *gui, Widget form)
{
	int		ac;
	Arg		al[30];
	Widget		frame;
	Widget		frame_box;
	Widget		label;
	XmString	label_string;
	Widget		radiobox;
	Widget		text;
	Widget		toggle;
/*
** Type box
*/
	make_frame("Include Directories", form, &frame_box, &frame);
/*
** Just one big text field
*/
	label_string = XmStringCreateSimple("-I. ");

	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,      XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,   XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,     XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,    XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNlabelString,    	label_string); ac++;

	text = XmCreateText(frame, "textfield", al, ac);
	XtManageChild(text);

	XmStringFree(label_string);

	gui->includes = text;

	return(frame_box);
}

/*-----------------------------------------------------------------
** Function:
**	target_lib_box
**
*/

Widget target_lib_box(ib_target_gui *gui, Widget form)
{
	int		ac;
	Arg		al[30];
	Widget		checkbox;
	Widget		frame;
	Widget		frame_box;
	Widget		label;
	XmString	label_string;
	Widget		rowcol;
	Widget		text;
	Widget		toggle;
	Widget		workarea;
/*
** Type box
*/
	make_frame("Libraries", form, &frame_box, &frame);

	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,      XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,   XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,     XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,    XmATTACH_FORM); ac++;

	workarea = XmCreateForm(frame, "form", al, ac);
	XtManageChild(workarea);
/*
** Input field
*/
	label_string = XmStringCreateSimple("");

	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,      XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,   XmATTACH_NONE); ac++;
	XtSetArg(al[ac], XmNleftAttachment,     XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,    XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNlabelString,    	label_string); ac++;

	text = XmCreateText(workarea, "textfield", al, ac);
	XtManageChild(text);

	XmStringFree(label_string);

	gui->local_libs = text;
/*
** Library options
*/
	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,      XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNtopWidget,      	text); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,   XmATTACH_NONE); ac++;
	XtSetArg(al[ac], XmNleftAttachment,     XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,    XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNpacking,    	XmPACK_COLUMN); ac++;
	XtSetArg(al[ac], XmNnumColumns,    	2); ac++;

	checkbox = XmCreateSimpleCheckBox(workarea, "checkbox", al, ac);
	XtManageChild(checkbox);
/*
** Xlib check button
*/
	label_string = XmStringCreateSimple("X Lib");

	ac = 0;
	XtSetArg(al[ac], XmNlabelString,	label_string); ac++;

	toggle = XmCreateToggleButton(checkbox, "toggle", al, ac);
	XtManageChild(toggle);

	XmStringFree(label_string);
	gui->use_x_lib = toggle;
/*
** Xtlib check button
*/
	label_string = XmStringCreateSimple("X Toolkit Lib");

	ac = 0;
	XtSetArg(al[ac], XmNlabelString,	label_string); ac++;

	toggle = XmCreateToggleButton(checkbox, "toggle", al, ac);
	XtManageChild(toggle);

	XmStringFree(label_string);
	gui->use_xt_lib = toggle;
/*
** Xawlib check button
*/
	label_string = XmStringCreateSimple("X Athena Widget Lib");

	ac = 0;
	XtSetArg(al[ac], XmNlabelString,	label_string); ac++;

	toggle = XmCreateToggleButton(checkbox, "toggle", al, ac);
	XtManageChild(toggle);

	XmStringFree(label_string);
	gui->use_xaw_lib = toggle;
/*
** Xmlib check button
*/
	label_string = XmStringCreateSimple("X Motif Lib");

	ac = 0;
	XtSetArg(al[ac], XmNlabelString,	label_string); ac++;

	toggle = XmCreateToggleButton(checkbox, "toggle", al, ac);
	XtManageChild(toggle);

	XmStringFree(label_string);
	gui->use_xm_lib = toggle;
/*
** F77 check button
*/
	label_string = XmStringCreateSimple("F77 Libs");

	ac = 0;
	XtSetArg(al[ac], XmNlabelString,	label_string); ac++;

	toggle = XmCreateToggleButton(checkbox, "toggle", al, ac);
	XtManageChild(toggle);

	XmStringFree(label_string);
	gui->use_f77_lib = toggle;
/*
** GL check button
*/
	label_string = XmStringCreateSimple("Graphics Libs (SGI GL)");

	ac = 0;
	XtSetArg(al[ac], XmNlabelString,	label_string); ac++;

	toggle = XmCreateToggleButton(checkbox, "toggle", al, ac);
	XtManageChild(toggle);

	XmStringFree(label_string);
	gui->use_gl_lib = toggle;
/*
** Return boxed widget
*/
	return(frame_box);
}


/*-----------------------------------------------------------------
** Function:
**	target_source_box
**
*/

Widget target_source_box(ib_target_gui *gui, Widget form)
{
	int		ac;
	Arg		al[30];
	Widget		button;
	Widget		frame;
	Widget		frame_box;
	Widget		rowcol;
	Widget		separator;
	Widget		sourcelist;
	Widget		workarea;
        extern void 	get_source_files(ib_target_gui *gui,
				Widget w, Widget source_list);
	extern void	target_addlist_cb(Widget w, ib_target_gui *target,
		XtPointer cbs);
	extern void	target_dellist_cb(Widget w, ib_target_gui *target,
		XtPointer cbs);
	extern void	target_resetlist_cb(Widget w, ib_target_gui *target,
		XtPointer cbs);
/*
** Frame box
*/
	make_frame("Source Files", form, &frame_box, &frame);

	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,      XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,   XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,     XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,    XmATTACH_FORM); ac++;

	workarea = XmCreateForm(frame, "form", al, ac);
	XtManageChild(workarea);
/*
** Rowcol for control buttons
*/
	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,      XmATTACH_NONE); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,   XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,     XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,    XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNorientation,    	XmHORIZONTAL); ac++;
	XtSetArg(al[ac], XmNpacking,    	XmPACK_COLUMN); ac++;
	XtSetArg(al[ac], XmNadjustLast,    	False); ac++;

	rowcol = XmCreateRowColumn(workarea, "rowcol", al, ac);
	XtManageChild(rowcol);
/*
** Add, Del, and Reset buttons
*/
	ac = 0;

	button = XmCreatePushButtonGadget(rowcol, "Add", al, ac);
	XtManageChild(button);
	XtAddCallback(button, XmNactivateCallback,
		(XtCallbackProc) target_addlist_cb, (XtPointer) gui);

	button = XmCreatePushButtonGadget(rowcol, "Delete", al, ac);
	XtManageChild(button);
	XtAddCallback(button, XmNactivateCallback,
		(XtCallbackProc) target_dellist_cb, (XtPointer) gui);

	button = XmCreatePushButtonGadget(rowcol, "Reset", al, ac);
	XtManageChild(button);
	XtAddCallback(button, XmNactivateCallback,
		(XtCallbackProc) target_resetlist_cb, (XtPointer) gui);
/*
** Separator
*/
	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,      XmATTACH_NONE); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,   XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNbottomWidget,   	rowcol); ac++;
	XtSetArg(al[ac], XmNleftAttachment,     XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,    XmATTACH_FORM); ac++;

	separator = XmCreateSeparator(workarea, "separator", al, ac);
	XtManageChild(separator);
/*
** Source file list widget
*/
	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,      XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,   XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNbottomWidget,   	separator); ac++;
	XtSetArg(al[ac], XmNleftAttachment,     XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,    XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNselectionPolicy,    XmMULTIPLE_SELECT); ac++;

	sourcelist = XmCreateScrolledList(workarea, "list", al, ac);
	XtManageChild(sourcelist);

	get_source_files(gui, workarea, sourcelist);

	gui->file_list = sourcelist;

	return(frame_box);
}

/*-----------------------------------------------------------------
** Function:
**	set_ib_target_gui
**
*/

void set_ib_target_gui(ib_target *t)
{
	ib_target_gui	*gui;
	XmString	item;
	int		i;

	if (t->gui->top_level == NULL) return;

	gui = t->gui;

	XmToggleButtonSetState(gui->is_directory, False, False);
	XmToggleButtonSetState(gui->is_library, False, False);
	XmToggleButtonSetState(gui->is_program, False, False);

	switch (t->type) {
	case IB_PROGRAM:
		XmToggleButtonSetState(gui->is_program, True, False);
		break;
	case IB_LIBRARY:
		XmToggleButtonSetState(gui->is_library, True, False);
		break;
	case IB_DIRECTORY:
		XmToggleButtonSetState(gui->is_directory, True, False);
		break;
	default:
		XmToggleButtonSetState(gui->is_program, True, False);
		break;
	}

	XmToggleButtonSetState(gui->use_lint, t->use_lint, False);
	XmToggleButtonSetState(gui->use_install, t->use_install, False);
	XmToggleButtonSetState(gui->use_man, t->use_man, False);
	XmToggleButtonSetState(gui->use_depend, t->use_depend, False);

	XmToggleButtonSetState(gui->use_x_lib, t->use_x_lib, False);
	XmToggleButtonSetState(gui->use_xt_lib, t->use_xt_lib, False);
	XmToggleButtonSetState(gui->use_xaw_lib, t->use_xaw_lib, False);
	XmToggleButtonSetState(gui->use_xm_lib, t->use_xm_lib, False);
	XmToggleButtonSetState(gui->use_f77_lib, t->use_f77_lib, False);
	XmToggleButtonSetState(gui->use_gl_lib, t->use_gl_lib, False);

	XmTextSetString(gui->man_text, t->man_text);
	XmTextSetString(gui->install_text, t->install_text);
	XmTextSetString(gui->depend_text, t->depend_text);

	XmTextSetString(gui->comments_text, t->comments_text);

	XmTextSetString(gui->includes, t->include_dirs);

	XmTextSetString(gui->local_libs, t->local_libs);

	if (t->source_list != NULL) {
		XmListDeleteAllItems(t->gui->file_list);

		for (i=0;i<t->source_count;i++) {
			item = XmStringCreateSimple(t->source_list[i]);

			XmListAddItem(t->gui->file_list, item, 0);

			XmStringFree(item);
		}
	}
}

/*-----------------------------------------------------------------
** Function:
**	get_ib_target_gui
**
*/

void get_ib_target_gui(ib_target *t)
{
	ib_target_gui	*gui;
	XmString	*item_list;
	char		*text;
	int		i;
	int		count;

	if (t->gui->top_level == NULL) return;

	gui = t->gui;

	if (XmToggleButtonGetState(gui->is_directory)) {
		t->type = IB_DIRECTORY;
	} else if (XmToggleButtonGetState(gui->is_library)) {
		t->type = IB_LIBRARY;
	} else {
		t->type = IB_PROGRAM;
	}

	t->use_lint = XmToggleButtonGetState(gui->use_lint);
	t->use_man = XmToggleButtonGetState(gui->use_man);
	t->use_install = XmToggleButtonGetState(gui->use_install);
	t->use_depend = XmToggleButtonGetState(gui->use_depend);

	t->use_x_lib = XmToggleButtonGetState(gui->use_x_lib);
	t->use_xt_lib = XmToggleButtonGetState(gui->use_xt_lib);
	t->use_xaw_lib = XmToggleButtonGetState(gui->use_xaw_lib);
	t->use_xm_lib = XmToggleButtonGetState(gui->use_xm_lib);
	t->use_f77_lib = XmToggleButtonGetState(gui->use_f77_lib);
	t->use_gl_lib = XmToggleButtonGetState(gui->use_gl_lib);

	strcpy(t->man_text, XmTextGetString(gui->man_text));
	strcpy(t->install_text, XmTextGetString(gui->install_text));
	strcpy(t->depend_text, XmTextGetString(gui->depend_text));

	t->comments_text = XmTextGetString(gui->comments_text);

	strcpy(t->include_dirs, XmTextGetString(gui->includes));

	strcpy(t->local_libs, XmTextGetString(gui->local_libs));

	XtVaGetValues(t->gui->file_list,
		XmNitems,	&item_list,
		XmNitemCount,	&count,
		NULL);

	if (t->source_list != NULL) {
		for(i=0;i<t->source_count;i++) {
			free(t->source_list[i]);
		}
		free(t->source_list);
		t->source_list = NULL;
	}

	t->source_count = count;

	if (count > 1) {
		t->source_list = (char **) malloc(sizeof (char *) * count);
	} else {
		t->source_list = (char **) malloc(sizeof (char *) * 1);
	}

	for (i=0;i<count;i++) {
		XmStringGetLtoR(item_list[i], XmSTRING_DEFAULT_CHARSET,
			&text);

		t->source_list[i] = (char *)
			malloc(sizeof(char) * (strlen(text)+1));

		strcpy(t->source_list[i], text);
		XtFree(text);
	}
}
