/*
** Program:
**	IMAKE BUILDER
**
** Purpose:
**	Constructs proper Imake Imakefiles
**
** Notes:
**-------------------------------------------------------------------------
**
**                    Copyright (c) 1994 James R. Dyvig
** 
** Permission to use, copy, modify, and distribute this software and its
** documentation for educational, research, and non-profit purposes
** without fee is hereby granted, provided that the above copyright notice
** appear in all copies and that both that copyright notice and this
** permission notice appear in supporting documentation.
** 
** Permission to incorporate this software into commercial products can be
** obtained with the written consent of the author.
** 
** This software is provided AS IS with no warranties of any kind.  The
** author shall have no liability with respect to the infringement of
** copyrights, trade secrets or any patents by this file or any part
** thereof.  In no event will the author be liable for any lost revenue or
** profits or other special, indirect and consequential damages.
**
**-------------------------------------------------------------------------
*/
#include <stdio.h>
#include <stdlib.h>
#include <Xm/Form.h>

#include "ib_project.h"

String	fallbacks[] = {
	"*foreground:                   white",
	"*background:                   cyan4",
	"*textfield*marginHeight:       2",
	"*fontList: -*-helvetica-bold-r-normal-*-14-*-*-*-*-*-*-*",
	"*grouplabel*fontList: -*-helvetica-bold-o-normal-*-14-*-*-*-*-*-*-*",
	"*titlelabel*fontList: -*-helvetica-bold-r-normal-*-18-*-*-*-*-*-*-*",
	"*titlelabel*foreground:	yellow",
	NULL};

ib_project	*project = NULL;

int main(int argc, char **argv)
{
	int		ac;		/* Argument count */
	Arg		al[30];		/* Argument list */
	XtAppContext	appContext;	/* Application Context for topLevel */
	Widget		form;		/* Background form to place menu */
	Widget		menuBar;	/* Menu bar */
	Widget		topLevel;	/* Base widget for app */
	Widget		workArea;
	extern Widget	main_menubar(Widget form);
	extern void	load_imakefile(ib_project *project);
	extern void	check_for_registration(void);

#ifndef NOREGFORM
/*
** The first thing to do is to check for user registration
*/
	check_for_registration();
#endif
/*
** Create the top level shell and set the title and its size
*/
	topLevel = XtAppInitialize(&appContext, "IB", NULL, 0,
		(Cardinal *) &argc, argv,
		fallbacks, NULL, 0);
/*
** Set attributes of the top level shell.  We set the height to simulate
** possible other widgets in the GUI.  The width is automatically adjusted
** to the width of the menu buttons.
*/
	ac = 0;
	XtSetArg(al[ac], XmNheight,	400); ac++;
	XtSetArg(al[ac], XmNtitle,	"Imake Builder"); ac++;
	XtSetValues(topLevel, al, ac);
/*
** Create a form widget to put the menu bar on
*/
	ac = 0;
	form = XmCreateForm(topLevel, "form", al, ac);
	XtManageChild(form);

	menuBar = main_menubar(form);
/*
** Create work area
*/
	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNtopWidget,		menuBar); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_FORM); ac++;

	workArea = XmCreateForm(form, "form", al, ac);
	XtManageChild(workArea);
/*
** Add a project
*/
	project = new_ib_project(workArea);
/*
** Realize the top level widget and enter the event loop
*/
	XtRealizeWidget(topLevel);

	load_imakefile(project);

	XtAppMainLoop(appContext);
}


/*-----------------------------------------------------------------
** Function:	check_for_registration
**
*/

void check_for_registration(void)
{
	char	buffer[120];
	FILE	*fp;
	char	*home_dir;
	char	namecheck[120];
	char	reg_filename[120];
	int	status;
/*
** Construct the name of the registration file in the home directory
*/
	home_dir = getenv("HOME");
	if (home_dir == NULL) {
		fprintf(stderr,"Could not find home directory\n");
		return;
	}

	sprintf(reg_filename, "%s/.regform", home_dir);
/*
** Parse each line, looking for the proper application name
*/
	fp = fopen(reg_filename, "r");
	if (fp) {
		sprintf(namecheck, "Application: %s %s\n",
			"ImakeBuilder", IB_VERSION);

		while (fgets(buffer, 120, fp)) {
			if (strcmp(buffer, namecheck) == 0) {
				fclose(fp);
				return;
			}
		}
		fclose(fp);
	}
/*
** Try to get the user to register
*/
	sprintf(buffer, "regform -a ImakeBuilder -v %s -s jdyvig@photon.com",
		IB_VERSION);
	
	status = system(buffer);

	if (status != 0) {
		fprintf(stderr,
		"Could not register.  Please e-mail jdyvig@photon.com\n");
	}
}
