/*-------------------------------------------------------------------------
**
**                    Copyright (c) 1994 James R. Dyvig
** 
** Permission to use, copy, modify, and distribute this software and its
** documentation for educational, research, and non-profit purposes
** without fee is hereby granted, provided that the above copyright notice
** appear in all copies and that both that copyright notice and this
** permission notice appear in supporting documentation.
** 
** Permission to incorporate this software into commercial products can be
** obtained with the written consent of the author.
** 
** This software is provided AS IS with no warranties of any kind.  The
** author shall have no liability with respect to the infringement of
** copyrights, trade secrets or any patents by this file or any part
** thereof.  In no event will the author be liable for any lost revenue or
** profits or other special, indirect and consequential damages.
**
**-------------------------------------------------------------------------
*/
#include <stdio.h>
#include <stdlib.h>
#include <Xm/MessageB.h>

#include "ib_project.h"
#include "ib_prefs.h"
#include "utils.h"

/*-----------------------------------------------------------------
** Function:
**	really_quitCB
**
*/

void really_quitCB(Widget w, XtPointer value, XmAnyCallbackStruct *cbs)
{
	Widget			topLevel;
/*
** Locate the top level shell and destroy it.  This is not necessary,
** but it does keep our foot in the door for better cleanup policies
** in the future.
*/
	topLevel = w;
	while (topLevel && !XtIsTopLevelShell(topLevel))
		topLevel = XtParent(topLevel);
	
	XtDestroyWidget(topLevel);

	exit(0);
}

/*-----------------------------------------------------------------
** Function:
**	quitCB
**
*/

void quitCB(Widget w, XtPointer value, XmAnyCallbackStruct *cbs)
{
	Widget			dialog;
	XmString		message;
	extern ib_project	*project;
	extern void		really_quitCB(Widget w, XtPointer value,
					XmAnyCallbackStruct *cbs);
/*
** Make sure the user has saved something
*/
	if (! project->saved) {
		dialog = XmCreateQuestionDialog(w, "Info", NULL, 0);
		message = XmStringCreateSimple("Imakefile not saved");

		XtVaSetValues(dialog,
			XmNmessageString,       message,
			XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL,
			NULL);
		
		XmStringFree(message);

		XtAddCallback(dialog, XmNokCallback,
			(XtCallbackProc) really_quitCB, NULL);
		XtManageChild(dialog);
		XtPopup(XtParent(dialog), XtGrabNone);
	} else {
		really_quitCB(w, NULL, NULL);
	}
}

/*-----------------------------------------------------------------
** Function:
**	headerCB
**
*/

void headerCB(Widget w, XtPointer value, XmAnyCallbackStruct *cbs)
{
	extern ib_project	*project;
	extern void ib_edit_header(Widget w, ib_project *project);

	ib_edit_header(w, project);
}

/*-----------------------------------------------------------------
** Function:
**	customCB
**
*/

void customCB(Widget w, XtPointer value, XmAnyCallbackStruct *cbs)
{
	extern ib_project	*project;
	extern void ib_edit_custom_options(Widget w, ib_project *project);

	ib_edit_custom_options(w, project);
}

/*-----------------------------------------------------------------
** Function:
**	prefCB
**
*/

void prefCB(Widget w, XtPointer value, XmAnyCallbackStruct *cbs)
{
	extern ib_project	*project;

	set_ib_prefs_gui(project->prefs);
	show_ib_prefs_gui(project->prefs->gui);
}

/*-----------------------------------------------------------------
** Function:
**	buildCB
**
*/

void buildCB(Widget w, XtPointer value, XmAnyCallbackStruct *cbs)
{
	extern ib_project	*project;
	extern void		write_imakefile(ib_project *project);
	
	write_imakefile(project);
}

/*-----------------------------------------------------------------
** Function:
**	makeCB
**
*/

void makeCB(Widget w, XtPointer value, XmAnyCallbackStruct *cbs)
{
	char			args[MAXCHAR];
	char			buffer[MAXCHAR];
	char			bootstrap[10];
	ib_prefs		*p;
	extern ib_project	*project;

	p = project->prefs;

	switch (p->bootstrap) {
	case IB_IMAKE:
		strcpy(bootstrap, "imake");
		sprintf(args,"%s -I%s", p->bootstrap_imake_text,
			p->config_location);
		break;
	case IB_XMKMF:
		strcpy(bootstrap, "xmkmf");
		strcpy(args, p->bootstrap_xmkmf_text);
		break;
	case IB_CUSTOM:
		strcpy(bootstrap, "");
		strcpy(args, p->bootstrap_custom_text);
		break;
	}

	sprintf(buffer, "%s %s", bootstrap, args);
	system(buffer);
}

/*-----------------------------------------------------------------
** Function:
**	dependCB
**
*/

void dependCB(Widget w, XtPointer value, XmAnyCallbackStruct *cbs)
{
	system("make depend");
}

/*-----------------------------------------------------------------
** Function:
**	version_helpCB
**
*/

#include "version_help.h"

void version_helpCB(Widget w, XtPointer value, XmAnyCallbackStruct *cbs)
{
	extern char	version_help_text[];

	show_text_block(w, "Imake Builder", "HELP:  Version",
		version_help_text);
}

/*-----------------------------------------------------------------
** Function:
**	overview_helpCB
**
*/

#include "overview_help.h"

void overview_helpCB(Widget w, XtPointer value, XmAnyCallbackStruct *cbs)
{
	extern char	overview_help_text[];

	show_text_block(w, "Imake Builder", "HELP:  Overview",
		overview_help_text);
}

/*-----------------------------------------------------------------
** Function:
**	edit_imakefileCB
**
*/

void edit_imakefileCB(Widget w, XtPointer value, XmAnyCallbackStruct *cbs)
{
	edit_file(w, "Imakefile");
}

/*-----------------------------------------------------------------
** Function:
**	view_configCB
**
*/

void view_configCB(Widget w, XtPointer value, XmAnyCallbackStruct *cbs)
{
}


/*-----------------------------------------------------------------
** Function:
**	regCB
**
*/

void regCB(Widget w, XtPointer value, XmAnyCallbackStruct *cbs)
{
	char	buffer[80];
	int	status;

	sprintf(buffer, "regform -a ImakeBuilder -v %s -s jdyvig@photon.com",
		IB_VERSION);
	
	status = system(buffer);

	if (status != 0) {
		warning(w, "Could not find regform");
	}
}


/*-----------------------------------------------------------------
** Function:
**	bugsCB
**
*/

void bugsCB(Widget w, XtPointer value, XmAnyCallbackStruct *cbs)
{
	char	buffer[80];
	int	status;

	sprintf(buffer,
		"regform -a ImakeBuilder -v %s -s jdyvig@photon.com -c &",
		IB_VERSION);
	
	status = system(buffer);

	if (status != 0) {
		warning(w, "Could not find regform");
	}
}
