/*-------------------------------------------------------------------------
**
**                    Copyright (c) 1994 James R. Dyvig
** 
** Permission to use, copy, modify, and distribute this software and its
** documentation for educational, research, and non-profit purposes
** without fee is hereby granted, provided that the above copyright notice
** appear in all copies and that both that copyright notice and this
** permission notice appear in supporting documentation.
** 
** Permission to incorporate this software into commercial products can be
** obtained with the written consent of the author.
** 
** This software is provided AS IS with no warranties of any kind.  The
** author shall have no liability with respect to the infringement of
** copyrights, trade secrets or any patents by this file or any part
** thereof.  In no event will the author be liable for any lost revenue or
** profits or other special, indirect and consequential damages.
**
**-------------------------------------------------------------------------
*/
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdlib.h>

#include <Xm/BulletinB.h>
#include <Xm/CascadeB.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/MessageB.h>
#include <Xm/PushB.h>
#include <Xm/PushBG.h>
#include <Xm/RowColumn.h>
#include <Xm/Separator.h>
#include <Xm/Text.h>

#include "utils.h"

Widget	stext;

/*-----------------------------------------------------------------
** Function:
**	make_menu_separator
**
*/

Widget make_menu_separator(Widget menu)
{
	int		ac;
	Arg		al[30];
	Widget		item;

	ac = 0;

	item = XmCreateSeparator(menu, "menu_separator", al, ac);
	XtManageChild(item);

	return(item);
}

/*-----------------------------------------------------------------
** Function:
**	make_menu_item
**
*/

Widget make_menu_item(char *name, void *callback, XtPointer data, Widget menu)
{
	int		ac;
	Arg		al[30];
	Widget		item;
	XmString	name_string;

	name_string = XmStringCreateSimple(name);

	ac = 0;
	XtSetArg(al[ac], XmNlabelString,	name_string); ac++;

	item = XmCreatePushButton(menu, name, al, ac);
	XtManageChild(item);

	if (callback != NULL) {
		XtAddCallback(item, XmNactivateCallback, callback, data);
	}

	XtSetSensitive(item, True);

	XmStringFree(name_string);

	return(item);
}

/*-----------------------------------------------------------------
** Function:
**	make_menu
**
*/

Widget make_menu(char *name, Widget menu_bar)
{
	int		ac;
	Arg		al[30];
	Widget		cascade;
	Widget		menu;
	XmString	name_string;

	ac = 0;
	menu = XmCreatePulldownMenu(menu_bar, name, al, ac);

	name_string = XmStringCreateSimple(name);

	ac = 0;
	XtSetArg(al[ac], XmNsubMenuId,		menu); ac++;
	XtSetArg(al[ac], XmNlabelString,	name_string); ac++;

	cascade = XmCreateCascadeButton(menu_bar, name, al, ac);
	XtManageChild(cascade);

	XmStringFree(name_string);

	return(menu);
}

/*-----------------------------------------------------------------
** Function:
**	make_help_menu
**
*/

Widget make_help_menu(char *name, Widget menu_bar)
{
	int		ac;
	Arg		al[30];
	Widget		cascade;
	Widget		menu;
	XmString	name_string;

	ac = 0;

	menu = XmCreatePulldownMenu(menu_bar, name, al, ac);

	name_string = XmStringCreateSimple(name);

	ac = 0;
	XtSetArg(al[ac], XmNsubMenuId,		menu); ac++;
	XtSetArg(al[ac], XmNlabelString,	name_string); ac++;

	cascade = XmCreateCascadeButton(menu_bar, name, al, ac);
	XtManageChild(cascade);

	XmStringFree(name_string);

	ac = 0;
	XtSetArg(al[ac], XmNmenuHelpWidget,      cascade); ac++;

	XtSetValues(menu_bar, al, ac);

	return(menu);
}

/*-----------------------------------------------------------------
** Function:
**	make_frame
**
*/

void make_frame(char *name, Widget parent, Widget *form, Widget *frame)
{
	int		ac;
	Arg		al[30];
	Widget		bulletinboard;
	Widget		label;
	XmString	label_string;

	ac = 0;

	*form = XmCreateForm(parent, "form", al, ac);
	XtManageChild(*form);

	label_string = XmStringCreateSimple(name);

	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,      XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNlabelString,      	label_string); ac++;
	XtSetArg(al[ac], XmNx,    		20); ac++;

	label = XmCreateLabel(*form, "grouplabel", al, ac);
	XtManageChild(label);

	XmStringFree(label_string);

	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,     XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNheight,     	9); ac++;
	XtSetArg(al[ac], XmNwidth,     		10); ac++;

	bulletinboard = XmCreateBulletinBoard(*form, "bulletin_board", al, ac);
	XtManageChild(bulletinboard);

	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNtopWidget,		bulletinboard); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,   XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,     XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,    XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNmarginWidth,	4); ac++;
	XtSetArg(al[ac], XmNmarginHeight,       7); ac++;

	*frame = XmCreateFrame(*form, "frame", al, ac);
	XtManageChild(*frame);
}

/*-----------------------------------------------------------------
** Function:
**	warning
**
*/

void warning(Widget w, char *info_string)
{
        XmString        message;
        Widget          dialog;

        dialog = XmCreateInformationDialog(w, "Info", NULL, 0);
        message = XmStringCreateSimple(info_string);

        XtVaSetValues(dialog,
                XmNmessageString,       message,
                XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL,
                NULL);

        XmStringFree(message);

        XtManageChild(dialog);
        XtPopup(XtParent(dialog), XtGrabNone);
}

/*-----------------------------------------------------------------
** Function:
**	del_window
**
*/

void del_window(Widget w, Widget shell, XtPointer cbs)
{
	XtDestroyWidget(shell);
}

/*-----------------------------------------------------------------
** Function:
**	new_window
**
*/

Widget new_window(Widget w,char *app_title,char *title, int width, int height)
{
	int		ac;
	Arg		al[30];
	Widget		button;
	Widget		form;
	Widget		frame;
	Widget		frame_box;
	Widget		separator;
	Widget		shell;
	Widget		topLevel;
	Widget		workarea;
/*
** Find the top level shell to spawn of another window from
*/
	topLevel = w;
	while (topLevel && !XtIsTopLevelShell(topLevel))
		topLevel = XtParent(topLevel);
/*
** Create a new shell
*/
	shell = XtVaAppCreateShell("IB", "IB",
		topLevelShellWidgetClass, XtDisplay(topLevel),
		NULL, 0);

	ac = 0;
	XtSetArg(al[ac], XmNwidth,	width); ac++;
	XtSetArg(al[ac], XmNheight,	height); ac++;
	XtSetArg(al[ac], XmNtitle,	app_title); ac++;

	XtSetValues(shell, al, ac);
/*
** Create a form to work from
*/
	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_FORM); ac++;

	form = XmCreateForm(shell, "form", al, ac);
	XtManageChild(form);
/*
** Create control buttons on bottom
*/
	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_NONE); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_NONE); ac++;

	button = XmCreatePushButtonGadget(form, "Dismiss", al, ac);
	XtManageChild(button);

	XtAddCallback(button, XmNactivateCallback,
		(XtCallbackProc) del_window, (XtPointer) shell);
/*
** Separator
*/
	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_NONE); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,	XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNbottomWidget,	button); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_FORM); ac++;

	separator = XmCreateSeparator(form, "separator", al, ac);
	XtManageChild(separator);
/*
** Working Form
*/
	make_frame(title, form, &frame_box, &frame);

	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,	XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNbottomWidget,	separator); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_FORM); ac++;

	XtSetValues(frame_box, al, ac);

	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_FORM); ac++;

	workarea = XmCreateForm(frame, "form", al, ac);
	XtManageChild(workarea);


	XtRealizeWidget(shell);
	return(workarea);
}

/*-----------------------------------------------------------------
** Function:
**	new_edit_window
**
*/

Widget new_edit_window(Widget w,char *app_title,char *title,
	int width, int height, XtCallbackProc callback, XtPointer data)
{
	int		ac;
	Arg		al[30];
	Widget		button;
	Widget		form;
	Widget		frame;
	Widget		frame_box;
	Widget		separator;
	Widget		shell;
	Widget		topLevel;
	Widget		workarea;
/*
** Find the top level shell to spawn of another window from
*/
	topLevel = w;
	while (topLevel && !XtIsTopLevelShell(topLevel))
		topLevel = XtParent(topLevel);
/*
** Create a new shell
*/
	shell = XtVaAppCreateShell("IB", "IB",
		topLevelShellWidgetClass, XtDisplay(topLevel),
		NULL, 0);

	ac = 0;
	XtSetArg(al[ac], XmNwidth,	width); ac++;
	XtSetArg(al[ac], XmNheight,	height); ac++;
	XtSetArg(al[ac], XmNtitle,	app_title); ac++;

	XtSetValues(shell, al, ac);
/*
** Create a form to work from
*/
	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_FORM); ac++;

	form = XmCreateForm(shell, "form", al, ac);
	XtManageChild(form);
/*
** Create control buttons on bottom
*/
	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_NONE); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_NONE); ac++;

	button = XmCreatePushButtonGadget(form, "Save", al, ac);
	XtManageChild(button);

	XtAddCallback(button, XmNactivateCallback,
		callback, (XtPointer) data);

	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_NONE); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNleftWidget,		button); ac++;
	XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_NONE); ac++;

	button = XmCreatePushButtonGadget(form, "Cancel", al, ac);
	XtManageChild(button);

	XtAddCallback(button, XmNactivateCallback,
		(XtCallbackProc) del_window, (XtPointer) shell);
/*
** Separator
*/
	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_NONE); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,	XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNbottomWidget,	button); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_FORM); ac++;

	separator = XmCreateSeparator(form, "separator", al, ac);
	XtManageChild(separator);
/*
** Working Form
*/
	make_frame(title, form, &frame_box, &frame);

	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,	XmATTACH_WIDGET); ac++;
	XtSetArg(al[ac], XmNbottomWidget,	separator); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_FORM); ac++;

	XtSetValues(frame_box, al, ac);

	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_FORM); ac++;

	workarea = XmCreateForm(frame, "form", al, ac);
	XtManageChild(workarea);


	XtRealizeWidget(shell);
	return(workarea);
}

/*-----------------------------------------------------------------
** Function:
**	save_imakefileCB
**
*/
void save_imakefileCB(Widget w, XtPointer value, XtPointer cbs)
{
	FILE		*fp;
	char		*text;
	Widget		text_area;
	Widget		topLevel;
	extern Widget	stext;

	topLevel = w;
	while (topLevel && !XtIsTopLevelShell(topLevel))
		topLevel = XtParent(topLevel);

	fp = fopen("Imakefile", "w");
	if (!fp) {
		warning(topLevel, "Could not save Imakefile");
		return;
	}

	text = XmTextGetString(stext);

	fwrite(text, sizeof(char), strlen(text), fp);

	fclose(fp);

	free(text);

	del_window(topLevel, topLevel, NULL);
}

/*-----------------------------------------------------------------
** Function:
**	edit_file
**
*/

void edit_file(Widget w, char *filename)
{
	int		ac;
	Arg		al[30];
	char		buffer[200];
	char		*contents;
	Widget		form;
	Widget		scrolled_text;
/*
** Read in the text file
*/
	contents = read_file(filename);
	if (contents == NULL) {
		warning(w, "Could not open file");
		return;
	}
/*
** Open a new window and place the text in it
*/
	sprintf(buffer,"EDIT:  %s", filename);
	form = new_edit_window(w, "Imake Builder", buffer, 600, 300,
		(XtCallbackProc) save_imakefileCB, NULL);

	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNeditMode,		XmMULTI_LINE_EDIT); ac++;
	XtSetArg(al[ac], XmNeditable,		True); ac++;

	scrolled_text = XmCreateScrolledText(form, "textblock", al, ac);
	XtManageChild(scrolled_text);

	stext = scrolled_text;

	XmTextSetString(scrolled_text, contents);

	free(contents);
}

/*-----------------------------------------------------------------
** Function:
**	show_file
**
*/

void show_file(Widget w, char *app_title, char *title, char *filename)
{
	int		ac;
	Arg		al[30];
	char		*contents;
	Widget		form;
	Widget		scrolled_text;
/*
** Read in the text file
*/
	contents = read_file(filename);
	if (contents == NULL) {
		warning(w, "Could not open help file");
		return;
	}
/*
** Open a new window and place the text in it
*/
	form = new_window(w, "Imake Builder", "HELP:  Version", 600, 300);

	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNeditMode,		XmMULTI_LINE_EDIT); ac++;
	XtSetArg(al[ac], XmNeditable,		False); ac++;

	scrolled_text = XmCreateScrolledText(form, "textblock", al, ac);
	XtManageChild(scrolled_text);

	XmTextSetString(scrolled_text, contents);

	free(contents);
}

/*-----------------------------------------------------------------
** Function:
**	show_text_block
**
*/

void show_text_block(Widget w, char *app_title, char *title, char *textblock)
{
	int		ac;
	Arg		al[30];
	Widget		form;
	Widget		scrolled_text;
/*
** Open a new window and place the text in it
*/
	form = new_window(w, app_title, title, 600, 300);

	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNbottomAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNleftAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNrightAttachment,	XmATTACH_FORM); ac++;
	XtSetArg(al[ac], XmNeditMode,		XmMULTI_LINE_EDIT); ac++;
	XtSetArg(al[ac], XmNeditable,		False); ac++;

	scrolled_text = XmCreateScrolledText(form, "textblock", al, ac);
	XtManageChild(scrolled_text);

	XmTextSetString(scrolled_text, textblock);
}

/*-----------------------------------------------------------------
** Function:
**	read_file
**
*/

char *read_file(char *filename)
{
	char		*contents;
	FILE		*fp;
	int		len;
	struct stat	stat_val;

	if (stat(filename, &stat_val) == 0) {
		len = stat_val.st_size;
		if ((fp = fopen(filename, "r")) != NULL) {
			contents = (char *) malloc(sizeof(char) * len + 10);
			contents[0] = '\0';

			fread(contents, sizeof(char), len, fp);
			contents[len] = '\0';
			fclose(fp);

			return(contents);
		}
	}

	return (NULL);
}

/*--------------------------------------------------------------
** Function:
**	get_file_path(char *env_var, char *filename)
**
** Purpose:
**	Searches the current directory and the directory
**	specified in the environment variable named in "env_var"
**	and returns the full pathname to the desired file.
**	If the file is not found, a NULL is returned.
*/

char *get_file_path(char *env_var, char *filename)
{
	FILE		*fp;
	char		*lib_dir;
	static char	pathname[512];
	struct stat     statinfo;
/*
** Try to get the comet lib directory
*/
	lib_dir = getenv(env_var);
/*
** Environment variable not set or set to blank
*/
	if (! lib_dir || strcmp(lib_dir,"") == 0) {
		strcpy(pathname, filename);
/*
** Environment variable set, so look there
*/
	} else {
		strcpy(pathname,lib_dir);
		strcat(pathname,"/");
		strcat(pathname,filename);
	}
/*
** Now look check for existence, looking in the current directory
** first, of course.
*/
	if ( stat(filename, &statinfo) == 0 ) {
		strcpy(pathname,filename);
		return (pathname);
	} else if ( stat(pathname, &statinfo) == 0 ) {
		return (pathname);
	}

	return (NULL);
}
